@@ -1,13 +1,34 @@
-#!/bin/sh
+#!/bin/env bash
 
 #
 # Test libmongocrypt's Debian packaging scripts.
 #
-# Supported/used environment variables:
-#   IS_PATCH    If "true", this is an Evergreen patch build.
+# Supported options:
+#   --is-patch={true,false}
+#       If "true", this is an Evergreen patch build. (Default 'false')
+#   --arch=<arch>
+#       If specified, sets the "--arch" option for debootstrap.
 
-set -o xtrace
-set -o errexit
+set -euxo pipefail
+
+IS_PATCH=false
+_dbs_args=()
+
+for arg in "$@"; do
+  case $arg in
+    --arch=*)
+      a="${arg#*=}"
+      _dbs_args+=(--arch "$a")
+      ;;
+    --is-patch=*)
+      IS_PATCH="${arg#*=}"
+      ;;
+    *)
+      echo "Unknown argument '$arg'"
+      exit 1
+      ;;
+  esac
+done
 
 on_exit () {
   if [ -e ./unstable-chroot/debootstrap/debootstrap.log ]; then
@@ -39,17 +60,19 @@ fi
 
 cd ..
 
+_dbs_args+=(unstable)
+
 git clone https://salsa.debian.org/installer-team/debootstrap.git debootstrap.git
 export DEBOOTSTRAP_DIR=`pwd`/debootstrap.git
-sudo -E ./debootstrap.git/debootstrap unstable ./unstable-chroot/ http://cdn-aws.deb.debian.org/debian
+sudo -E ./debootstrap.git/debootstrap "${_dbs_args[@]}" ./unstable-chroot/ http://cdn-aws.deb.debian.org/debian
 cp -a libmongocrypt ./unstable-chroot/tmp/
 sudo chroot ./unstable-chroot /bin/bash -c "(set -o xtrace && \
-  apt-get install -y build-essential git-buildpackage fakeroot debhelper cmake curl ca-certificates libssl-dev pkg-config libbson-dev && \
+  apt-get install -y build-essential git-buildpackage fakeroot debhelper cmake curl ca-certificates libssl-dev pkg-config libbson-dev libintelrdfpmath-dev && \
   chown -R root:root /tmp/libmongocrypt && \
   cd /tmp/libmongocrypt && \
   git clean -fdx && \
   git reset --hard HEAD && \
-  cmake -DRELEASE_BRANCH_REF=origin/r1.5 -P ./cmake/GetVersion.cmake > VERSION_CURRENT 2>&1 && \
+  python3 etc/calc_release_version.py > VERSION_CURRENT && \
   git add --force VERSION_CURRENT && \
   git commit VERSION_CURRENT -m 'Set current version' && \
   LANG=C /bin/bash ./debian/build_snapshot.sh && \
