@@ -43,7 +43,7 @@ repositories {
 }
 
 group = "org.mongodb"
-version = "1.5.3-SNAPSHOT"
+version = "1.7.2-SNAPSHOT"
 description = "MongoDB client-side crypto support"
 
 java {
@@ -66,20 +66,29 @@ dependencies {
 /*
  * Git version information
  */
-val gitVersion: String by lazy {
+
+// Returns a String representing the output of `git describe`
+val gitDescribe by lazy {
     val describeStdOut = ByteArrayOutputStream()
     exec {
         commandLine = listOf("git", "describe", "--tags", "--always", "--dirty")
         standardOutput = describeStdOut
     }
-    val gv: String = describeStdOut.toString().trim()
-    gv.subSequence(gv.toCharArray().indexOfFirst { it.isDigit() }, gv.length).toString()
+    describeStdOut.toString().trim()
 }
 
-val gitHash: String by lazy {
+val isJavaTag by lazy { gitDescribe.startsWith("java") }
+val gitVersion by lazy { gitDescribe.subSequence(gitDescribe.toCharArray().indexOfFirst { it.isDigit() }, gitDescribe.length).toString() }
+
+val defaultDownloadRevision: String by lazy {
+    val gitCommandLine = if (gitVersion == version) {
+        listOf("git", "rev-list", "-n", "1", gitVersion)
+    } else {
+        listOf("git", "rev-parse", "HEAD")
+    }
     val describeStdOut = ByteArrayOutputStream()
     exec {
-        commandLine = listOf("git", "rev-parse", "HEAD")
+        commandLine = gitCommandLine
         standardOutput = describeStdOut
     }
     describeStdOut.toString().trim()
@@ -90,20 +99,20 @@ val gitHash: String by lazy {
  */
 val jnaDownloadsDir = "$buildDir/jnaLibs/downloads/"
 val jnaResourcesDir = "$buildDir/jnaLibs/resources/"
-val jnaLibsPath: String = System.getProperty("jnaLibsPath", "${jnaResourcesDir}${com.sun.jna.Platform.RESOURCE_PREFIX}")
+val jnaLibPlatform: String = if (com.sun.jna.Platform.RESOURCE_PREFIX.startsWith("darwin")) "darwin" else com.sun.jna.Platform.RESOURCE_PREFIX
+val jnaLibsPath: String = System.getProperty("jnaLibsPath", "${jnaResourcesDir}${jnaLibPlatform}")
 val jnaResources: String = System.getProperty("jna.library.path", jnaLibsPath)
 
 // Download jnaLibs that match the git to jnaResourcesBuildDir
-val revision: String = System.getProperty("gitRevision", if (gitVersion == version) gitVersion else gitHash)
-val downloadUrl: String = "https://mciuploads.s3.amazonaws.com/libmongocrypt/java/$revision/libmongocrypt-java.tar.gz"
+val downloadRevision: String = System.getProperties().computeIfAbsent("gitRevision") { k -> defaultDownloadRevision }.toString()
+val downloadUrl: String = "https://mciuploads.s3.amazonaws.com/libmongocrypt/java/$downloadRevision/libmongocrypt-java.tar.gz"
 
 val jnaMapping: Map<String, String> = mapOf(
     "rhel-62-64-bit" to "linux-x86-64",
-    "rhel-67-s390x" to "linux-s390x",
+    "rhel72-zseries-test" to "linux-s390x",
     "rhel-71-ppc64el" to "linux-ppc64le",
     "ubuntu1604-arm64" to "linux-aarch64",
     "windows-test" to "win32-x86-64",
-    "macos_x86_64" to "darwin-x86-64",
     "macos" to "darwin"
 )
 
@@ -120,10 +129,11 @@ tasks.register<Copy>("unzipJava") {
         listOf("${it}/nocrypto/**/libmongocrypt.so", "${it}/nocrypto/**/libmongocrypt.dylib", "${it}/nocrypto/**/mongocrypt.dll" )
     })
     eachFile {
-        path = "${jnaMapping.get(path.substringBefore("/"))}/${name}"
+        path = "${jnaMapping[path.substringBefore("/")]}/${name}"
     }
     into(jnaResourcesDir)
     mustRunAfter("downloadJava")
+    tasks.withType<ProcessResources>().forEach { t -> t.mustRunAfter("unzipJava") }
 
     doLast {
         println("jna.library.path contents: \n  ${fileTree(jnaResourcesDir).files.joinToString(",\n  ")}")
@@ -254,37 +264,57 @@ tasks.javadoc {
     }
 }
 
-tasks.register("publishSnapshots") {
+tasks.register("publishToSonatype") {
     group = "publishing"
-    description = "Publishes snapshots to Sonatype"
-    if (version.toString().endsWith("-SNAPSHOT")) {
-        dependsOn("downloadJnaLibs")
-        dependsOn(tasks.withType<PublishToMavenRepository>())
-    }
-}
-
-tasks.register("publishArchives") {
-    group = "publishing"
-    description = "Publishes a release and uploads to Sonatype / Maven Central"
+    description = """Publishes to Sonatype.
+        |
+        | - If the version string ends with SNAPSHOT then publishes to the Snapshots repo.
+        |   Note: Uses the JNA libs from the current build.
+        |
+        | - If is a release then publishes the release to maven central staging.
+        |   A release is when the current git tag is prefixed with java (eg: java-1.7.0)
+        |   AND the git tag version matches the version the build.gradle.kts.
+        |   Note: Uses the JNA libs from the associated tag.
+        |   Eg: Tag java-1.7.0 will use the JNA libs created by the 1.7.0 release tag.
+        |
+        | To override the JNA library downloaded use -DgitRevision=<git hash>
+    """.trimMargin()
+    val isSnapshot = version.toString().endsWith("-SNAPSHOT")
+    val isRelease = isSnapshot || (isJavaTag && gitVersion == version)
 
     doFirst {
-        if (gitVersion != version) {
-            val cause = """
-                | Version mismatch:
-                | =================
+        if (isSnapshot && isJavaTag) {
+                throw GradleException("""
+                | Invalid Release
+                | ===============
                 |
-                | $version != $gitVersion
+                | Version: $version 
+                | GitVersion: $gitVersion
+                | isJavaTag: $isJavaTag
+                |
+                |""".trimMargin())
+        }
+
+        if (isRelease) {
+            println("Publishing: ${project.name} : $gitVersion")
+        } else {
+            println("""
+                | Not a Java release:
+                |
+                | Version:
+                | ========
+                |
+                | $gitDescribe
                 |
                 | The project version does not match the git tag.
-                |""".trimMargin()
-            throw GradleException(cause)
-        } else {
-            println("Publishing: ${project.name} : $gitVersion")
+                |""".trimMargin())
         }
     }
 
-    if (gitVersion == version) {
-        dependsOn(tasks.withType<PublishToMavenRepository>())
+    if (isRelease) {
+        dependsOn("downloadJnaLibs")
+        finalizedBy(tasks.withType<PublishToMavenRepository>())
+        tasks.withType<PublishToMavenRepository>().forEach { t -> t.mustRunAfter("downloadJnaLibs", "downloadJava", "unzipJava") }
     }
 }
 
