@@ -102,11 +102,10 @@ public class CAPI {
     public static class mongocrypt_kms_ctx_t extends PointerType {
     }
 
-
     /**
      * Returns the version string x.y.z for libmongocrypt.
      *
-     * @param len, an optional length of the returned string. May be NULL.
+     * @param len an optional length of the returned string. May be NULL.
      * @return the version string x.y.z for libmongocrypt.
      */
     public static native cstring
@@ -227,7 +226,7 @@ public class CAPI {
      * Get the error message associated with a status, or an empty string.
      *
      * @param status The status object.
-     * @param len, an optional length of the returned string. May be NULL.
+     * @param len an optional length of the returned string. May be NULL.
      * @return An error message or an empty string.
      */
     public static native cstring
@@ -321,6 +320,41 @@ public class CAPI {
                                    mongocrypt_hash_fn sha_256,
                                    Pointer ctx);
 
+    /**
+     * Set a crypto hook for the AES256-CTR operations.
+     *
+     * @param crypt The @ref mongocrypt_t object.
+     * @param aes_256_ctr_encrypt The crypto callback function for encrypt
+     * operation.
+     * @param aes_256_ctr_decrypt The crypto callback function for decrypt
+     * operation.
+     * @param ctx A context passed as an argument to the crypto callback
+     * every invocation.
+     * @return A boolean indicating success. If false, an error status is set.
+     * Retrieve it with @ref mongocrypt_status
+     *
+     */
+    public static native boolean
+    mongocrypt_setopt_aes_256_ctr (mongocrypt_t crypt,
+            mongocrypt_crypto_fn aes_256_ctr_encrypt,
+            mongocrypt_crypto_fn aes_256_ctr_decrypt,
+            Pointer ctx);
+
+    /**
+     * Set a crypto hook for the RSASSA-PKCS1-v1_5 algorithm with a SHA-256 hash.
+     *
+     * <p>See: https://tools.ietf.org/html/rfc3447#section-8.2</p>
+     *
+     * <p>Note: this function has the wrong name. It should be:
+     * mongocrypt_setopt_crypto_hook_sign_rsassa_pkcs1_v1_5</p>
+     *
+     * @param crypt The @ref mongocrypt_t object.
+     * @param sign_rsaes_pkcs1_v1_5 The crypto callback function.
+     * @param sign_ctx A context passed as an argument to the crypto callback
+     * every invocation.
+     * @return A boolean indicating success. If false, an error status is set.
+     * Retrieve it with @ref mongocrypt_status
+     */
     public static native boolean
     mongocrypt_setopt_crypto_hook_sign_rsaes_pkcs1_v1_5(
             mongocrypt_t crypt,
@@ -397,6 +431,152 @@ public class CAPI {
     public static native void
     mongocrypt_setopt_use_need_kms_credentials_state (mongocrypt_t crypt);
 
+
+    /**
+     * Set a local EncryptedFieldConfigMap for encryption.
+     *
+     * @param crypt The @ref mongocrypt_t object.
+     * @param encryptedFieldConfigMap A BSON document representing the EncryptedFieldConfigMap
+     * supplied by the user. The keys are collection namespaces and values are
+     * EncryptedFieldConfigMap documents. The viewed data copied. It is valid to
+     * destroy @p efc_map with @ref mongocrypt_binary_destroy immediately after.
+     * @return A boolean indicating success. If false, an error status is set.
+     * Retrieve it with @ref mongocrypt_status
+     */
+    public static native boolean
+    mongocrypt_setopt_encrypted_field_config_map (mongocrypt_t crypt, mongocrypt_binary_t encryptedFieldConfigMap);
+
+    /**
+     * Opt-into skipping query analysis.
+     *
+     * <p>If opted in:
+     * <ul>
+     *     <li>The crypt_shared shared library will not attempt to be loaded.</li>
+     *     <li>A mongocrypt_ctx_t will never enter the MONGOCRYPT_CTX_NEED_MARKINGS state.</li>
+     * </ul>
+     *
+     * @param crypt The @ref mongocrypt_t object to update
+     * @since 1.5
+     */
+    public static native void
+    mongocrypt_setopt_bypass_query_analysis (mongocrypt_t crypt);
+
+    /**
+     * Set the contention factor used for explicit encryption.
+     * The contention factor is only used for indexed Queryable Encryption.
+     *
+     * @param ctx The @ref mongocrypt_ctx_t object.
+     * @param contention_factor the contention factor
+     * @return A boolean indicating success. If false, an error status is set.
+     * Retrieve it with @ref mongocrypt_ctx_status.
+     * @since 1.5
+     */
+    public static native boolean
+    mongocrypt_ctx_setopt_contention_factor (mongocrypt_ctx_t ctx, long contention_factor);
+
+    /**
+     * Set the index key id to use for Queryable Encryption explicit encryption.
+     *
+     * If the index key id not set, the key id from @ref mongocrypt_ctx_setopt_key_id is used.
+     *
+     * @param ctx The @ref mongocrypt_ctx_t object.
+     * @param key_id The binary corresponding to the _id (a UUID) of the data key to use from
+     *               the key vault collection. Note, the UUID must be encoded with RFC-4122 byte order.
+     *               The viewed data is copied. It is valid to destroy key_id with @ref mongocrypt_binary_destroy immediately after.
+     * @return A boolean indicating success. If false, an error status is set.
+     * Retrieve it with @ref mongocrypt_ctx_status
+     * @since 1.5
+     */
+    public static native boolean
+    mongocrypt_ctx_setopt_index_key_id (mongocrypt_ctx_t ctx, mongocrypt_binary_t key_id);
+
+    /**
+     * Append an additional search directory to the search path for loading
+     * the crypt_shared dynamic library.
+     *
+     * @param crypt The @ref mongocrypt_t object to update
+     * @param path A null-terminated sequence of bytes for the search path. On
+     * some filesystems, this may be arbitrary bytes. On other filesystems, this may
+     * be required to be a valid UTF-8 code unit sequence. If the leading element of
+     * the path is the literal string "$ORIGIN", that substring will be replaced
+     * with the directory path containing the executable libmongocrypt module. If
+     * the path string is literal "$SYSTEM", then libmongocrypt will defer to the
+     * system's library resolution mechanism to find the crypt_shared library.
+     *
+     * <p>If no crypt_shared dynamic library is found in any of the directories
+     * specified by the search paths loaded here, @ref mongocrypt_init() will still
+     * succeed and continue to operate without crypt_shared.</p>
+     *
+     * <p>The search paths are searched in the order that they are appended. This
+     * allows one to provide a precedence in how the library will be discovered. For
+     * example, appending known directories before appending "$SYSTEM" will allow
+     * one to supersede the system's installed library, but still fall-back to it if
+     * the library wasn't found otherwise. If one does not ever append "$SYSTEM",
+     * then the system's library-search mechanism will never be consulted.</p>
+     *
+     * <p>If an absolute path to the library is specified using @ref mongocrypt_setopt_set_crypt_shared_lib_path_override,
+     * then paths appended here will have no effect.</p>
+     * @since 1.5
+     */
+    public static native void
+    mongocrypt_setopt_append_crypt_shared_lib_search_path (mongocrypt_t crypt, cstring path);
+
+    /**
+     * Set a single override path for loading the crypt_shared dynamic library.
+     * @param crypt The @ref mongocrypt_t object to update
+     * @param path A null-terminated sequence of bytes for a path to the crypt_shared
+     * dynamic library. On some filesystems, this may be arbitrary bytes. On other
+     * filesystems, this may be required to be a valid UTF-8 code unit sequence. If
+     * the leading element of the path is the literal string `$ORIGIN`, that
+     * substring will be replaced with the directory path containing the executable
+     * libmongocrypt module.
+     *
+     * <p>This function will do no IO nor path validation. All validation will
+     * occur during the call to @ref mongocrypt_init.</p>
+     * <p>If a crypt_shared library path override is specified here, then no paths given
+     * to @ref mongocrypt_setopt_append_crypt_shared_lib_search_path will be consulted when
+     * opening the crypt_shared library.</p>
+     * <p>If a path is provided via this API and @ref mongocrypt_init fails to
+     * initialize a valid crypt_shared library instance for the path specified, then
+     * the initialization of mongocrypt_t will fail with an error.</p>
+     * @since 1.5
+     */
+    public static native void
+    mongocrypt_setopt_set_crypt_shared_lib_path_override(mongocrypt_t crypt, cstring path);
+
+    /**
+     * Set the query type to use for Queryable Encryption explicit encryption.
+     * The query type is only used for indexed Queryable Encryption.
+     *
+     * @param ctx The @ref mongocrypt_ctx_t object.
+     * @param query_type the query type
+     * @return A boolean indicating success. If false, an error status is set.
+     * Retrieve it with @ref mongocrypt_ctx_status
+     */
+    public static native boolean
+    mongocrypt_ctx_setopt_query_type (mongocrypt_ctx_t ctx, cstring query_type, int len);
+
+    /**
+     * Set options for explicit encryption with the "rangePreview" algorithm.
+     * NOTE: The RangePreview algorithm is experimental only. It is not intended for
+     * public use.
+     *
+     * opts is a BSON document of the form:
+     * {
+     *    "min": Optional<BSON value>,
+     *    "max": Optional<BSON value>,
+     *    "sparsity": Int64,
+     *    "precision": Optional<Int32>
+     * }
+     *
+     * @param ctx The @ref mongocrypt_ctx_t object.
+     * @param opts BSON.
+     * @return A boolean indicating success. If false, an error status is set.
+     * @since 1.7
+     */
+    public static native boolean
+    mongocrypt_ctx_setopt_algorithm_range (mongocrypt_ctx_t ctx, mongocrypt_binary_t opts);
+
     /**
      * Initialize new @ref mongocrypt_t object.
      *
@@ -426,6 +606,20 @@ public class CAPI {
     public static native void
     mongocrypt_destroy(mongocrypt_t crypt);
 
+    /**
+     * Obtain a nul-terminated version string of the loaded crypt_shared dynamic library,
+     * if available.
+     *
+     * If no crypt_shared was successfully loaded, this function returns NULL.
+     *
+     * @param crypt The mongocrypt_t object after a successful call to mongocrypt_init.
+     * @param len an optional length of the returned string. May be NULL.
+     *
+     * @return A nul-terminated string of the dynamically loaded crypt_shared library.
+     * @since 1.5
+     */
+    public static native cstring
+    mongocrypt_crypt_shared_lib_version_string (mongocrypt_t crypt, Pointer len);
 
     /**
      * Call in response to the MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS state
@@ -471,6 +665,24 @@ public class CAPI {
     mongocrypt_ctx_setopt_key_alt_name (mongocrypt_ctx_t ctx,
                                         mongocrypt_binary_t key_alt_name);
 
+    /**
+     * Set the keyMaterial to use for encrypting data.
+     *
+     * <p>
+     * Pass the binary encoding of a BSON document like the following:
+     * <code>{ "keyMaterial" : (BSON BINARY value) }</code>
+     * </p>
+     *
+     * @param ctx The @ref mongocrypt_ctx_t object.
+     * @param key_material The data encryption key to use. The viewed data is
+     * copied. It is valid to destroy @p key_material with @ref
+     * mongocrypt_binary_destroy immediately after.
+     * @return A boolean indicating success. If false, an error status is set.
+     * Retrieve it with @ref mongocrypt_ctx_status
+     */
+    public static native boolean
+    mongocrypt_ctx_setopt_key_material (mongocrypt_ctx_t ctx, mongocrypt_binary_t key_material);
+
     /**
      * Set the algorithm used for encryption to either
      * deterministic or random encryption. This value
@@ -634,6 +846,52 @@ public class CAPI {
     mongocrypt_ctx_explicit_encrypt_init (mongocrypt_ctx_t ctx,
                                           mongocrypt_binary_t msg);
 
+    /**
+     * Explicit helper method to encrypt a Match Expression or Aggregate Expression.
+     * Contexts created for explicit encryption will not go through mongocryptd.
+     * Requires query_type to be "rangePreview".
+     * NOTE: The RangePreview algorithm is experimental only. It is not intended for
+     * public use.
+     *
+     * This method expects the passed-in BSON to be of the form:
+     * { "v" : FLE2RangeFindDriverSpec }
+     *
+     * FLE2RangeFindDriverSpec is a BSON document with one of these forms:
+     *
+     * 1. A Match Expression of this form:
+     *    {$and: [{<field>: {<op>: <value1>, {<field>: {<op>: <value2> }}]}
+     * 2. An Aggregate Expression of this form:
+     *    {$and: [{<op>: [<fieldpath>, <value1>]}, {<op>: [<fieldpath>, <value2>]}]
+     *
+     * may be $lt, $lte, $gt, or $gte.
+     *
+     * The value of "v" is expected to be the BSON value passed to a driver
+     * ClientEncryption.encryptExpression helper.
+     *
+     * Associated options for FLE 1:
+     * - @ref mongocrypt_ctx_setopt_key_id
+     * - @ref mongocrypt_ctx_setopt_key_alt_name
+     * - @ref mongocrypt_ctx_setopt_algorithm
+     *
+     * Associated options for Queryable Encryption:
+     * - @ref mongocrypt_ctx_setopt_key_id
+     * - @ref mongocrypt_ctx_setopt_index_key_id
+     * - @ref mongocrypt_ctx_setopt_contention_factor
+     * - @ref mongocrypt_ctx_setopt_query_type
+     * - @ref mongocrypt_ctx_setopt_algorithm_range
+     *
+     * An error is returned if FLE 1 and Queryable Encryption incompatible options
+     * are set.
+     *
+     * @param ctx A @ref mongocrypt_ctx_t.
+     * @param msg A @ref mongocrypt_binary_t the plaintext BSON value.
+     * @return A boolean indicating success.
+     * @since 1.7
+     */
+    public static native boolean
+    mongocrypt_ctx_explicit_encrypt_expression_init (mongocrypt_ctx_t ctx,
+                                                     mongocrypt_binary_t msg);
+
     /**
      * Initialize a context for decryption.
      *
@@ -656,6 +914,19 @@ public class CAPI {
     mongocrypt_ctx_explicit_decrypt_init (mongocrypt_ctx_t ctx,
                                           mongocrypt_binary_t msg);
 
+    /**
+     * Initialize a context to rewrap datakeys.
+     *
+     * Associated options {@link #mongocrypt_ctx_setopt_key_encryption_key(mongocrypt_ctx_t, mongocrypt_binary_t)}
+     *
+     * @param ctx The @ref mongocrypt_ctx_t object.
+     * @param filter The filter to use for the find command on the key vault collection to retrieve datakeys to rewrap.
+     * @return A boolean indicating success. If false, and error status is set.
+     * @since 1.5
+     */
+    public static native boolean
+    mongocrypt_ctx_rewrap_many_datakey_init (mongocrypt_ctx_t ctx, mongocrypt_binary_t filter);
+
 
     public static final int MONGOCRYPT_CTX_ERROR = 0;
     public static final int MONGOCRYPT_CTX_NEED_MONGO_COLLINFO = 1; /* run on main MongoClient */
@@ -666,6 +937,10 @@ public class CAPI {
     public static final int MONGOCRYPT_CTX_DONE = 6;
     public static final int MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS = 7; /* fetch/renew KMS credentials */
 
+    public static final int MONGOCRYPT_INDEX_TYPE_NONE = 1;
+    public static final int MONGOCRYPT_INDEX_TYPE_EQUALITY = 2;
+    public static final int MONGOCRYPT_QUERY_TYPE_EQUALITY = 1;
+
     /**
      * Get the current state of a context.
      *
