@@ -8,8 +8,9 @@ NODE_ARTIFACTS_PATH="${NODE_BINDINGS_PATH}/node-artifacts"
 NPM_CACHE_DIR="${NODE_ARTIFACTS_PATH}/npm"
 NPM_TMP_DIR="${NODE_ARTIFACTS_PATH}/tmp"
 BIN_DIR="$(pwd)/bin"
-NVM_WINDOWS_URL="https://github.com/coreybutler/nvm-windows/releases/download/1.1.9/nvm-noinstall.zip"
+NVM_WINDOWS_URL="https://github.com/coreybutler/nvm-windows/releases/download/1.1.10/nvm-noinstall.zip"
 NVM_URL="https://raw.githubusercontent.com/creationix/nvm/v0.38.0/install.sh"
+NPM_OPTIONS="${NPM_OPTIONS}"
 
 # create node artifacts path if needed
 mkdir -p "${NODE_ARTIFACTS_PATH}"
@@ -20,23 +21,15 @@ mkdir -p "${BIN_DIR}"
 # Add mongodb toolchain to path
 export PATH="$BIN_DIR:/opt/mongodbtoolchain/v2/bin:$PATH"
 
-# locate cmake
-if [ "$OS" == "Windows_NT" ]; then
-  CMAKE=/cygdrive/c/cmake/bin/cmake
-  if [ "$WINDOWS_32BIT" != "ON" ]; then
-      ADDITIONAL_CMAKE_FLAGS="-Thost=x64 -A x64"
-  fi
-else
-  chmod u+x ./.evergreen/find_cmake.sh
-  IGNORE_SYSTEM_CMAKE=1 . ./.evergreen/find_cmake.sh
-fi
+test -n "${NODE_NVM_USE_VERSION-}" || echo "Defaulting to using the current Node LTS Release. Set NODE_NVM_USE_VERSION to change."
+: "${NODE_NVM_USE_VERSION:="lts"}"
 
 # this needs to be explicitly exported for the nvm install below
 export NVM_DIR="${NODE_ARTIFACTS_PATH}/nvm"
 mkdir -p ${NVM_DIR}
 
 # install Node.js
-echo "Installing Node ${NODE_LTS_NAME}"
+echo "Installing Latest Node for Major Version ${NODE_NVM_USE_VERSION}"
 if [ "$OS" == "Windows_NT" ]; then
   set +o xtrace
 
@@ -57,10 +50,10 @@ root: $NVM_HOME
 path: $NVM_SYMLINK
 EOT
 
-  echo "Running: nvm install lts"
-  nvm install lts
-  echo "Running: nvm use lts"
-  nvm use lts
+  echo "Running: nvm install $NODE_NVM_USE_VERSION"
+  nvm install $NODE_NVM_USE_VERSION
+  echo "Running: nvm use $NODE_NVM_USE_VERSION"
+  nvm use $NODE_NVM_USE_VERSION
   echo "Running: npm install -g npm@8.3.1"
   npm install -g npm@8.3.1 # https://github.com/npm/cli/issues/4341
   set -o xtrace
@@ -71,10 +64,14 @@ else
   curl -o- $NVM_URL | bash
   [ -s "${NVM_DIR}/nvm.sh" ] && \. "${NVM_DIR}/nvm.sh"
 
-  echo "Running: nvm install --lts --latest-npm"
-  nvm install --lts --latest-npm
-  echo "Running: nvm use --lts"
-  nvm use --lts
+  declare _arg="$NODE_NVM_USE_VERSION"
+  if test "$NODE_NVM_USE_VERSION" = "lts"; then _arg="--lts"; fi
+
+  echo "Running: nvm install \"$_arg\" --latest-npm"
+  nvm install "$_arg" --latest-npm
+  echo "Running: nvm use \"$_arg\""
+  nvm use "$_arg"
+  node --version
 
   set -o xtrace
 fi
