@@ -3,6 +3,7 @@
 set -o errexit  # Exit the script with error if any of the commands fail
 
 echo "Setting up environment"
+export NPM_OPTIONS="${NPM_OPTIONS}"
 . ./.evergreen/setup_environment.sh
 
 # Handle the circular dependency when testing with a real client.
@@ -10,7 +11,7 @@ export MONGODB_CLIENT_ENCRYPTION_OVERRIDE="$(pwd)"
 
 # install node dependencies
 echo "Installing package dependencies (includes a static build)"
-. ./etc/build-static.sh
+bash ./etc/build-static.sh
 # npm install
 
 # Run tests
@@ -20,4 +21,4 @@ MONGODB_NODE_SKIP_LIVE_TESTS=true npm test
 
 # Run prebuild and deploy
 echo "Running prebuild and deploy"
-. ./.evergreen/prebuild.sh
+bash ./.evergreen/prebuild.sh
