@@ -110,7 +110,8 @@ Function MongoCrypt::Init(Napi::Env env) {
                     InstanceMethod("makeDataKeyContext", &MongoCrypt::MakeDataKeyContext),
                     InstanceMethod("makeRewrapManyDataKeyContext", &MongoCrypt::MakeRewrapManyDataKeyContext),
                     InstanceAccessor("status", &MongoCrypt::Status, nullptr),
-                    InstanceAccessor("cryptSharedLibVersionInfo", &MongoCrypt::CryptSharedLibVersionInfo, nullptr)
+                    InstanceAccessor("cryptSharedLibVersionInfo", &MongoCrypt::CryptSharedLibVersionInfo, nullptr),
+                    StaticValue("libmongocryptVersion", String::New(env, mongocrypt_version(nullptr)))
                   });
 }
 
@@ -581,19 +582,6 @@ Value MongoCrypt::MakeExplicitEncryptionContext(const CallbackInfo& info) {
                 throw TypeError::New(Env(), errorStringFromStatus(context.get()));
             }
         }
-
-        if (options.Has("indexKeyId")) {
-            Napi::Value indexKeyId = options["indexKeyId"];
-
-            if (!indexKeyId.IsBuffer()) {
-                throw TypeError::New(Env(), "`indexKeyId` must be a Buffer");
-            }
-
-            std::unique_ptr<mongocrypt_binary_t, MongoCryptBinaryDeleter> binary(BufferToBinary(indexKeyId.As<Uint8Array>()));
-            if (!mongocrypt_ctx_setopt_index_key_id(context.get(), binary.get())) {
-                throw TypeError::New(Env(), errorStringFromStatus(context.get()));
-            }
-        }
     }
 
     std::unique_ptr<mongocrypt_binary_t, MongoCryptBinaryDeleter> binaryValue(BufferToBinary(valueBuffer.As<Uint8Array>()));
