@@ -2,10 +2,10 @@
 
 const fs = require('fs');
 const path = require('path');
-const BSON = require('bson');
-const EJSON = require('bson').EJSON;
 const sinon = require('sinon');
 const mongodb = require('mongodb');
+const BSON = mongodb.BSON;
+const EJSON = BSON.EJSON;
 const requirements = require('./requirements.helper');
 const MongoNetworkTimeoutError = mongodb.MongoNetworkTimeoutError || mongodb.MongoTimeoutError;
 const MongoError = mongodb.MongoError;
@@ -66,6 +66,9 @@ class MockClient {
   }
 }
 
+const originalAccessKeyId = process.env.AWS_ACCESS_KEY_ID;
+const originalSecretAccessKey = process.env.AWS_SECRET_ACCESS_KEY;
+
 const AutoEncrypter = require('../lib/autoEncrypter')({ mongodb, stateMachine }).AutoEncrypter;
 describe('AutoEncrypter', function () {
   this.timeout(12000);
@@ -110,6 +113,157 @@ describe('AutoEncrypter', function () {
     sandbox.restore();
   });
 
+  describe('#constructor', function () {
+    context('when mongodb exports BSON (driver >= 4.9.0)', function () {
+      context('when a bson option is provided', function () {
+        const bson = Object.assign({}, BSON);
+        const encrypter = new AutoEncrypter(
+          {},
+          {
+            bson: bson,
+            kmsProviders: {
+              local: { key: Buffer.alloc(96) }
+            }
+          }
+        );
+
+        it('uses the bson option', function () {
+          expect(encrypter._bson).to.equal(bson);
+        });
+      });
+
+      context('when a bson option is not provided', function () {
+        const encrypter = new AutoEncrypter(
+          {},
+          {
+            kmsProviders: {
+              local: { key: Buffer.alloc(96) }
+            }
+          }
+        );
+
+        it('uses the mongodb exported BSON', function () {
+          expect(encrypter._bson).to.equal(BSON);
+        });
+      });
+
+      it('never uses bson from the topology', function () {
+        expect(() => {
+          new AutoEncrypter(
+            {},
+            {
+              kmsProviders: {
+                local: { key: Buffer.alloc(96) }
+              }
+            }
+          );
+        }).not.to.throw();
+      });
+    });
+
+    context('when mongodb does not export BSON (driver < 4.9.0)', function () {
+      context('when a bson option is provided', function () {
+        const bson = Object.assign({}, BSON);
+        const encrypter = new AutoEncrypter(
+          {},
+          {
+            bson: bson,
+            kmsProviders: {
+              local: { key: Buffer.alloc(96) }
+            }
+          }
+        );
+
+        it('uses the bson option', function () {
+          expect(encrypter._bson).to.equal(bson);
+        });
+      });
+
+      context('when a bson option is not provided', function () {
+        const mongoNoBson = { ...mongodb, BSON: undefined };
+        const AutoEncrypterNoBson = require('../lib/autoEncrypter')({
+          mongodb: mongoNoBson,
+          stateMachine
+        }).AutoEncrypter;
+
+        context('when the client has a topology', function () {
+          const client = new MockClient();
+          const encrypter = new AutoEncrypterNoBson(client, {
+            kmsProviders: {
+              local: { key: Buffer.alloc(96) }
+            }
+          });
+
+          it('uses the bson on the topology', function () {
+            expect(encrypter._bson).to.equal(client.topology.bson);
+          });
+        });
+
+        context('when the client does not have a topology', function () {
+          it('raises an error', function () {
+            expect(() => {
+              new AutoEncrypterNoBson({}, {});
+            }).to.throw(/bson/);
+          });
+        });
+      });
+    });
+
+    context('when using mongocryptd', function () {
+      const client = new MockClient();
+      const autoEncrypterOptions = {
+        mongocryptdBypassSpawn: true,
+        keyVaultNamespace: 'admin.datakeys',
+        logger: () => {},
+        kmsProviders: {
+          aws: { accessKeyId: 'example', secretAccessKey: 'example' },
+          local: { key: Buffer.alloc(96) }
+        }
+      };
+      const autoEncrypter = new AutoEncrypter(client, autoEncrypterOptions);
+
+      it('instantiates a mongo client on the auto encrypter', function () {
+        expect(autoEncrypter)
+          .to.have.property('_mongocryptdClient')
+          .to.be.instanceOf(mongodb.MongoClient);
+      });
+
+      it('sets the 3x legacy client options on the mongo client', function () {
+        expect(autoEncrypter).to.have.nested.property('_mongocryptdClient.s.options');
+        const options = autoEncrypter._mongocryptdClient.s.options;
+        expect(options).to.have.property('useUnifiedTopology', true);
+        expect(options).to.have.property('useNewUrlParser', true);
+      });
+
+      it('sets serverSelectionTimeoutMS to 10000ms', function () {
+        expect(autoEncrypter).to.have.nested.property('_mongocryptdClient.s.options');
+        const options = autoEncrypter._mongocryptdClient.s.options;
+        expect(options).to.have.property('serverSelectionTimeoutMS', 10000);
+      });
+
+      context('when mongocryptdURI is not specified', () => {
+        it('sets the ip address family to ipv4', function () {
+          expect(autoEncrypter).to.have.nested.property('_mongocryptdClient.s.options');
+          const options = autoEncrypter._mongocryptdClient.s.options;
+          expect(options).to.have.property('family', 4);
+        });
+      });
+
+      context('when mongocryptdURI is specified', () => {
+        it('does not set the ip address family to ipv4', function () {
+          const autoEncrypter = new AutoEncrypter(client, {
+            ...autoEncrypterOptions,
+            extraOptions: { mongocryptdURI: MongocryptdManager.DEFAULT_MONGOCRYPTD_URI }
+          });
+
+          expect(autoEncrypter).to.have.nested.property('_mongocryptdClient.s.options');
+          const options = autoEncrypter._mongocryptdClient.s.options;
+          expect(options).not.to.have.property('family', 4);
+        });
+      });
+    });
+  });
+
   it('should support `bypassAutoEncryption`', function (done) {
     const client = new MockClient();
     const autoEncrypter = new AutoEncrypter(client, {
@@ -130,23 +284,6 @@ describe('AutoEncrypter', function () {
     });
   });
 
-  context('when checking serverSelectionTimeoutMS on the mongocryptd client', function () {
-    const client = new MockClient();
-    const autoEncrypter = new AutoEncrypter(client, {
-      mongocryptdBypassSpawn: true,
-      keyVaultNamespace: 'admin.datakeys',
-      logger: () => {},
-      kmsProviders: {
-        aws: { accessKeyId: 'example', secretAccessKey: 'example' },
-        local: { key: Buffer.alloc(96) }
-      }
-    });
-
-    it('defaults to 10000', function () {
-      expect(autoEncrypter._mongocryptdClient.s.options.serverSelectionTimeoutMS).to.equal(10000);
-    });
-  });
-
   describe('state machine', function () {
     it('should decrypt mock data', function (done) {
       const input = readExtendedJsonToBuffer(`${__dirname}/data/encrypted-document.json`);
@@ -229,6 +366,85 @@ describe('AutoEncrypter', function () {
       });
     });
 
+    context('when no refresh function is provided', function () {
+      const accessKey = 'example';
+      const secretKey = 'example';
+
+      before(function () {
+        if (process.env.NPM_OPTIONS === '--no-optional') {
+          this.currentTest.skipReason = 'Cannot refresh credentials without sdk provider';
+          this.currentTest.skip();
+          return;
+        }
+        // After the entire suite runs, set the env back for the rest of the test run.
+        process.env.AWS_ACCESS_KEY_ID = accessKey;
+        process.env.AWS_SECRET_ACCESS_KEY = secretKey;
+      });
+
+      after(function () {
+        // After the entire suite runs, set the env back for the rest of the test run.
+        process.env.AWS_ACCESS_KEY_ID = originalAccessKeyId;
+        process.env.AWS_SECRET_ACCESS_KEY = originalSecretAccessKey;
+      });
+
+      it('should decrypt mock data with KMS credentials from the environment', function (done) {
+        const input = readExtendedJsonToBuffer(`${__dirname}/data/encrypted-document.json`);
+        const client = new MockClient();
+        const mc = new AutoEncrypter(client, {
+          keyVaultNamespace: 'admin.datakeys',
+          logger: () => {},
+          kmsProviders: {
+            aws: {}
+          }
+        });
+        mc.decrypt(input, (err, decrypted) => {
+          if (err) return done(err);
+          expect(decrypted).to.eql({ filter: { find: 'test', ssn: '457-55-5462' } });
+          done();
+        });
+      });
+    });
+
+    context('when no refresh function is provided and no optional sdk', function () {
+      const accessKey = 'example';
+      const secretKey = 'example';
+
+      before(function () {
+        if (!process.env.NPM_OPTIONS) {
+          this.currentTest.skipReason = 'With optional sdk installed credentials would be loaded.';
+          this.currentTest.skip();
+          return;
+        }
+        // After the entire suite runs, set the env back for the rest of the test run.
+        process.env.AWS_ACCESS_KEY_ID = accessKey;
+        process.env.AWS_SECRET_ACCESS_KEY = secretKey;
+      });
+
+      after(function () {
+        // After the entire suite runs, set the env back for the rest of the test run.
+        process.env.AWS_ACCESS_KEY_ID = originalAccessKeyId;
+        process.env.AWS_SECRET_ACCESS_KEY = originalSecretAccessKey;
+      });
+
+      it('errors without the optional sdk credential provider', function (done) {
+        const input = readExtendedJsonToBuffer(`${__dirname}/data/encrypted-document.json`);
+        const client = new MockClient();
+        const mc = new AutoEncrypter(client, {
+          keyVaultNamespace: 'admin.datakeys',
+          logger: () => {},
+          kmsProviders: {
+            aws: {}
+          }
+        });
+        mc.decrypt(input, err => {
+          expect(err.message).to.equal(
+            'client not configured with KMS provider necessary to decrypt'
+          );
+          done();
+        });
+      });
+    });
+
     it('should encrypt mock data', function (done) {
       const client = new MockClient();
       const mc = new AutoEncrypter(client, {
@@ -730,4 +946,8 @@ describe('AutoEncrypter', function () {
       this.mc.teardown(true, done);
     });
   });
+
+  it('should provide the libmongocrypt version', function () {
+    expect(AutoEncrypter.libmongocryptVersion).to.be.a('string');
+  });
 });
