@@ -22,22 +22,30 @@ if [ "Windows_NT" = "$OS" ]; then # Magic variable in cygwin
              "C:/python/Python37/python.exe"
              "C:/python/Python38/python.exe"
              "C:/python/Python39/python.exe"
-             "C:/python/Python310/python.exe")
+             "C:/python/Python310/python.exe"
+             "C:/python/Python311/python.exe")
     export CRYPT_SHARED_PATH=../crypt_shared/bin/mongo_crypt_v1.dll
     C:/python/Python310/python.exe drivers-evergreen-tools/.evergreen/mongodl.py --component crypt_shared \
       --version latest --out ../crypt_shared/
 elif [ "Darwin" = "$(uname -s)" ]; then
     export PYMONGOCRYPT_LIB=${MONGOCRYPT_DIR}/nocrypto/lib/libmongocrypt.dylib
-    PYTHONS=("python"   # Python 2.7 from brew
-             "python3"  # Python 3 from brew
-             "/System/Library/Frameworks/Python.framework/Versions/2.7/bin/python"
-             "/Library/Frameworks/Python.framework/Versions/3.4/bin/python3"
-             "/Library/Frameworks/Python.framework/Versions/3.5/bin/python3"
-             "/Library/Frameworks/Python.framework/Versions/3.6/bin/python3"
-             "/Library/Frameworks/Python.framework/Versions/3.7/bin/python3"
-             "/Library/Frameworks/Python.framework/Versions/3.8/bin/python3"
-             "/Library/Frameworks/Python.framework/Versions/3.9/bin/python3"
-             "/Library/Frameworks/Python.framework/Versions/3.10/bin/python3")
+    if [[ $(uname -m) == 'arm64' ]]; then
+      PYTHONS=("/Library/Frameworks/Python.framework/Versions/3.10/bin/python3"
+              )
+    else
+      PYTHONS=("python"   # Python 2.7 from brew
+               "python3"  # Python 3 from brew
+               "/System/Library/Frameworks/Python.framework/Versions/2.7/bin/python"
+               "/Library/Frameworks/Python.framework/Versions/3.4/bin/python3"
+               "/Library/Frameworks/Python.framework/Versions/3.5/bin/python3"
+               "/Library/Frameworks/Python.framework/Versions/3.6/bin/python3"
+               "/Library/Frameworks/Python.framework/Versions/3.7/bin/python3"
+               "/Library/Frameworks/Python.framework/Versions/3.8/bin/python3"
+               "/Library/Frameworks/Python.framework/Versions/3.9/bin/python3"
+               "/Library/Frameworks/Python.framework/Versions/3.10/bin/python3"
+               "/Library/Frameworks/Python.framework/Versions/3.11/bin/python3")
+    fi
+
     export CRYPT_SHARED_PATH="../crypt_shared/lib/mongo_crypt_v1.dylib"
     python3 drivers-evergreen-tools/.evergreen/mongodl.py --component crypt_shared \
       --version latest --out ../crypt_shared/
