@@ -14,11 +14,13 @@
 
 import copy
 
+
 from pymongocrypt.binary import (MongoCryptBinaryIn,
                                  MongoCryptBinaryOut)
 from pymongocrypt.binding import ffi, lib, _to_string
 from pymongocrypt.compat import (safe_bytearray_or_base64, str_to_bytes,
                                  unicode_type)
+from pymongocrypt.credentials import _ask_for_kms_credentials
 from pymongocrypt.errors import MongoCryptError
 from pymongocrypt.state_machine import MongoCryptCallback
 
@@ -71,6 +73,15 @@ class MongoCryptOptions(object):
             outgoing commands. Set `bypass_query_analysis` to use explicit
             encryption on indexed fields without the MongoDB Enterprise Advanced
             licensed crypt_shared library.
+          - `crypt_shared_lib_path`: Optional string path to the crypt_shared
+            library.
+          - `crypt_shared_lib_required`: Whether to require a crypt_shared
+            library.
+          - `bypass_encryption`: Whether to bypass encryption.
+
+        .. versionadded:: 1.3
+           ``crypt_shared_lib_path``, ``crypt_shared_lib_path``,
+           ``bypass_encryption`` parameters.
 
         .. versionadded:: 1.1
            Support for "azure" and "gcp" kms_providers.
@@ -89,30 +100,33 @@ class MongoCryptOptions(object):
             aws = kms_providers["aws"]
             if not isinstance(aws, dict):
                 raise ValueError("kms_providers['aws'] must be a dict")
-            if "accessKeyId" not in aws or "secretAccessKey" not in aws:
-                raise ValueError("kms_providers['aws'] must contain "
-                                 "'accessKeyId' and 'secretAccessKey'")
+            if len(aws):
+                if "accessKeyId" not in aws or "secretAccessKey" not in aws:
+                    raise ValueError("kms_providers['aws'] must contain "
+                                     "'accessKeyId' and 'secretAccessKey'")
 
         if 'azure' in kms_providers:
             azure = kms_providers["azure"]
             if not isinstance(azure, dict):
                 raise ValueError("kms_providers['azure'] must be a dict")
-            if 'clientId' not in azure or 'clientSecret' not in azure:
-                raise ValueError("kms_providers['azure'] must contain "
-                                 "'clientId' and 'clientSecret'")
+            if len(azure):
+                if 'clientId' not in azure or 'clientSecret' not in azure:
+                    raise ValueError("kms_providers['azure'] must contain "
+                                     "'clientId' and 'clientSecret'")
 
         if 'gcp' in kms_providers:
             gcp = kms_providers['gcp']
             if not isinstance(gcp, dict):
                 raise ValueError("kms_providers['gcp'] must be a dict")
-            if 'email' not in gcp or 'privateKey' not in gcp:
-                raise ValueError("kms_providers['gcp'] must contain "
-                                 "'email' and 'privateKey'")
-            if not isinstance(kms_providers['gcp']['privateKey'],
-                              (bytes, unicode_type)):
-                raise TypeError("kms_providers['gcp']['privateKey'] must "
-                                "be an instance of bytes or str "
-                                "(unicode in Python 2)")
+            if len(gcp):
+                if 'email' not in gcp or 'privateKey' not in gcp:
+                    raise ValueError("kms_providers['gcp'] must contain "
+                                     "'email' and 'privateKey'")
+                if not isinstance(kms_providers['gcp']['privateKey'],
+                                  (bytes, unicode_type)):
+                    raise TypeError("kms_providers['gcp']['privateKey'] must "
+                                    "be an instance of bytes or str "
+                                    "(unicode in Python 2)")
 
         if 'kmip' in kms_providers:
             kmip = kms_providers['kmip']
@@ -239,6 +253,11 @@ class MongoCrypt(object):
 
         if not self.__opts.bypass_encryption:
             lib.mongocrypt_setopt_append_crypt_shared_lib_search_path(self.__crypt, b"$SYSTEM")
+        on_demand_aws = 'aws' in kms_providers and not len(kms_providers['aws'])
+        on_demand_gcp = 'gcp' in kms_providers and not len(kms_providers['gcp'])
+        on_demand_azure = 'azure' in kms_providers and not len(kms_providers['azure'])
+        if any([on_demand_aws, on_demand_gcp, on_demand_azure]):
+            lib.mongocrypt_setopt_use_need_kms_credentials_state(self.__crypt)
 
         if not lib.mongocrypt_init(self.__crypt):
             self.__raise_from_status()
@@ -270,7 +289,11 @@ class MongoCrypt(object):
         """Cleanup resources."""
         if self.__crypt is None:
             return
-        lib.mongocrypt_destroy(self.__crypt)
+        # Since close is called by __del__, we need to be sure to guard
+        # against the case where global variables are set to None at
+        # interpreter shutdown, see PYTHON-3530.
+        if lib is not None:
+            lib.mongocrypt_destroy(self.__crypt)
         self.__crypt = None
 
     def __del__(self):
@@ -293,7 +316,7 @@ class MongoCrypt(object):
         :Returns:
           A :class:`EncryptionContext`.
         """
-        return EncryptionContext(self._create_context(), database, command)
+        return EncryptionContext(self._create_context(), self.__opts.kms_providers, database, command)
 
     def decryption_context(self, command):
         """Creates a context to use for decryption.
@@ -304,7 +327,7 @@ class MongoCrypt(object):
         :Returns:
           A :class:`DecryptionContext`.
         """
-        return DecryptionContext(self._create_context(), command)
+        return DecryptionContext(self._create_context(), self.__opts.kms_providers, command)
 
     def explicit_encryption_context(self, value, opts):
         """Creates a context to use for explicit encryption.
@@ -317,7 +340,8 @@ class MongoCrypt(object):
         :Returns:
           A :class:`ExplicitEncryptionContext`.
         """
-        return ExplicitEncryptionContext(self._create_context(), value, opts)
+        return ExplicitEncryptionContext(
+            self._create_context(), self.__opts.kms_providers, value, opts)
 
     def explicit_decryption_context(self, value):
         """Creates a context to use for explicit decryption.
@@ -329,7 +353,8 @@ class MongoCrypt(object):
         :Returns:
           A :class:`ExplicitDecryptionContext`.
         """
-        return ExplicitDecryptionContext(self._create_context(), value)
+        return ExplicitDecryptionContext(self._create_context(),
+            self.__opts.kms_providers, value)
 
     def data_key_context(self, kms_provider, opts=None):
         """Creates a context to use for key generation.
@@ -341,22 +366,39 @@ class MongoCrypt(object):
         :Returns:
           A :class:`DataKeyContext`.
         """
-        return DataKeyContext(self._create_context(), kms_provider, opts,
+        return DataKeyContext(self._create_context(), self.__opts.kms_providers, kms_provider, opts,
                               self.__callback)
 
+    def rewrap_many_data_key_context(self, filter, provider, master_key):
+        """Creates a context to use for rewrapping many data keys.
+
+        :Parameters:
+          - `filter`: A document used to filter the data keys.
+          - `provider`: (optional) The name of a different kms provider.
+          - `master_key`: Optional document for the given provider.
+            MUST have the fields corresponding to the
+            given provider as specified in master_key. master_key MUST NOT be
+            given if it is not applicable for the given provider.
+
+        :Returns:
+          A :class:`RewrapManyDataKeyContext`.
+        """
+        return RewrapManyDataKeyContext(self._create_context(), self.__opts.kms_providers, filter, provider, master_key, self.__callback)
+
 
 class MongoCryptContext(object):
-    __slots__ = ("__ctx",)
+    __slots__ = ("__ctx", "__kms_providers")
 
-    def __init__(self, ctx):
+    def __init__(self, ctx, kms_providers):
         """Abstracts libmongocrypt's mongocrypt_ctx_t type.
 
         :Parameters:
           - `ctx`: A mongocrypt_ctx_t. This MongoCryptContext takes ownership
             of the underlying mongocrypt_ctx_t.
-          - `database`: Optional, the name of the database.
+          - `kms_providers`: The KMS provider map.
         """
         self.__ctx = ctx
+        self.__kms_providers = kms_providers
 
     def _close(self):
         """Cleanup resources."""
@@ -407,6 +449,16 @@ class MongoCryptContext(object):
         if not lib.mongocrypt_ctx_mongo_done(self.__ctx):
             self._raise_from_status()
 
+    def ask_for_kms_credentials(self):
+        """Get on-demand kms credentials"""
+        return _ask_for_kms_credentials(self.__kms_providers)
+
+    def provide_kms_providers(self, providers):
+        """Provide a map of KMS providers."""
+        with MongoCryptBinaryIn(providers) as binary:
+            if not lib.mongocrypt_ctx_provide_kms_providers(self.__ctx, binary.bin):
+                self._raise_from_status()
+
     def kms_contexts(self):
         """Yields the MongoCryptKmsContexts."""
         ctx = lib.mongocrypt_ctx_next_kms_ctx(self.__ctx)
@@ -430,16 +482,17 @@ class MongoCryptContext(object):
 class EncryptionContext(MongoCryptContext):
     __slots__ = ("database",)
 
-    def __init__(self, ctx, database, command):
+    def __init__(self, ctx, kms_providers, database, command):
         """Abstracts libmongocrypt's mongocrypt_ctx_t type.
 
         :Parameters:
           - `ctx`: A mongocrypt_ctx_t. This MongoCryptContext takes ownership
             of the underlying mongocrypt_ctx_t.
+         - `kms_providers`: The KMS provider map.
           - `database`: Optional, the name of the database.
           - `command`: The BSON command to encrypt.
         """
-        super(EncryptionContext, self).__init__(ctx)
+        super(EncryptionContext, self).__init__(ctx, kms_providers)
         self.database = database
         try:
             with MongoCryptBinaryIn(command) as binary:
@@ -456,15 +509,16 @@ class EncryptionContext(MongoCryptContext):
 class DecryptionContext(MongoCryptContext):
     __slots__ = ()
 
-    def __init__(self, ctx, command):
+    def __init__(self, ctx, kms_providers, command):
         """Abstracts libmongocrypt's mongocrypt_ctx_t type.
 
         :Parameters:
           - `ctx`: A mongocrypt_ctx_t. This MongoCryptContext takes ownership
             of the underlying mongocrypt_ctx_t.
+          - `kms_providers`: The KMS provider map.
           - `command`: The encoded BSON command to decrypt.
         """
-        super(DecryptionContext, self).__init__(ctx)
+        super(DecryptionContext, self).__init__(ctx, kms_providers)
         try:
             with MongoCryptBinaryIn(command) as binary:
                 if not lib.mongocrypt_ctx_decrypt_init(ctx, binary.bin):
@@ -478,17 +532,18 @@ class DecryptionContext(MongoCryptContext):
 class ExplicitEncryptionContext(MongoCryptContext):
     __slots__ = ()
 
-    def __init__(self, ctx, value, opts):
+    def __init__(self, ctx, kms_providers, value, opts):
         """Abstracts libmongocrypt's mongocrypt_ctx_t type.
 
         :Parameters:
           - `ctx`: A mongocrypt_ctx_t. This MongoCryptContext takes ownership
             of the underlying mongocrypt_ctx_t.
+          - `kms_providers`: The KMS provider map.
           - `value`:  The encoded document to encrypt, which must be in the
             form { "v" : BSON value to encrypt }}.
           - `opts`: A :class:`ExplicitEncryptOpts`.
         """
-        super(ExplicitEncryptionContext, self).__init__(ctx)
+        super(ExplicitEncryptionContext, self).__init__(ctx, kms_providers)
         try:
             algorithm = str_to_bytes(opts.algorithm)
             if not lib.mongocrypt_ctx_setopt_algorithm(ctx, algorithm, -1):
@@ -504,11 +559,6 @@ class ExplicitEncryptionContext(MongoCryptContext):
                     if not lib.mongocrypt_ctx_setopt_key_alt_name(ctx, binary.bin):
                         self._raise_from_status()
 
-            if opts.index_key_id is not None:
-                with MongoCryptBinaryIn(opts.index_key_id) as binary:
-                    if not lib.mongocrypt_ctx_setopt_index_key_id(ctx, binary.bin):
-                        self._raise_from_status()
-
             if opts.query_type is not None:
                 qt = str_to_bytes(opts.query_type)
                 if not lib.mongocrypt_ctx_setopt_query_type(ctx, qt, -1):
@@ -518,9 +568,18 @@ class ExplicitEncryptionContext(MongoCryptContext):
                 if not lib.mongocrypt_ctx_setopt_contention_factor(ctx, opts.contention_factor):
                     self._raise_from_status()
 
+            if opts.range_opts is not None:
+                with MongoCryptBinaryIn(opts.range_opts) as range_opts:
+                    if not lib.mongocrypt_ctx_setopt_algorithm_range(ctx, range_opts.bin):
+                        self._raise_from_status()
+
             with MongoCryptBinaryIn(value) as binary:
-                if not lib.mongocrypt_ctx_explicit_encrypt_init(ctx, binary.bin):
-                    self._raise_from_status()
+                if opts.is_expression:
+                    if not lib.mongocrypt_ctx_explicit_encrypt_expression_init(ctx, binary.bin):
+                        self._raise_from_status()
+                else:
+                    if not lib.mongocrypt_ctx_explicit_encrypt_init(ctx, binary.bin):
+                        self._raise_from_status()
         except Exception:
             # Destroy the context on error.
             self._close()
@@ -530,15 +589,16 @@ class ExplicitEncryptionContext(MongoCryptContext):
 class ExplicitDecryptionContext(MongoCryptContext):
     __slots__ = ()
 
-    def __init__(self, ctx, value):
+    def __init__(self, ctx, kms_providers, value):
         """Abstracts libmongocrypt's mongocrypt_ctx_t type.
 
         :Parameters:
           - `ctx`: A mongocrypt_ctx_t. This MongoCryptContext takes ownership
             of the underlying mongocrypt_ctx_t.
+          - `kms_providers`: The KMS provider map.
           - `value`: The encoded BSON value to decrypt.
         """
-        super(ExplicitDecryptionContext, self).__init__(ctx)
+        super(ExplicitDecryptionContext, self).__init__(ctx, kms_providers)
 
         try:
             with MongoCryptBinaryIn(value) as binary:
@@ -554,17 +614,18 @@ class ExplicitDecryptionContext(MongoCryptContext):
 class DataKeyContext(MongoCryptContext):
     __slots__ = ()
 
-    def __init__(self, ctx, kms_provider, opts, callback):
+    def __init__(self, ctx, kms_providers, kms_provider, opts, callback):
         """Abstracts libmongocrypt's mongocrypt_ctx_t type.
 
         :Parameters:
           - `ctx`: A mongocrypt_ctx_t. This MongoCryptContext takes ownership
             of the underlying mongocrypt_ctx_t.
+          - `kms_providers`: The KMS provider map.
           - `kms_provider`: The KMS provider.
           - `opts`: An optional class:`DataKeyOpts`.
           - `callback`: A :class:`MongoCryptCallback`.
         """
-        super(DataKeyContext, self).__init__(ctx)
+        super(DataKeyContext, self).__init__(ctx, kms_providers)
         try:
             if kms_provider not in ['aws', 'gcp', 'azure', 'kmip', 'local']:
                 raise ValueError('unknown kms_provider: %s' % (kms_provider,))
@@ -614,6 +675,12 @@ class DataKeyContext(MongoCryptContext):
                                 ctx, binary.bin):
                             self._raise_from_status()
 
+            if opts.key_material:
+                with MongoCryptBinaryIn(opts.key_material) as binary:
+                    if not lib.mongocrypt_ctx_setopt_key_material(
+                            ctx, binary.bin):
+                        self._raise_from_status()
+
             if not lib.mongocrypt_ctx_datakey_init(ctx):
                 self._raise_from_status()
         except Exception:
@@ -698,3 +765,44 @@ class MongoCryptKmsContext(object):
         finally:
             lib.mongocrypt_status_destroy(status)
         raise exc
+
+
+class RewrapManyDataKeyContext(MongoCryptContext):
+    __slots__ = ()
+
+    def __init__(self, ctx, kms_providers, filter, provider, master_key,
+        callback):
+        """Abstracts libmongocrypt's mongocrypt_ctx_t type.
+
+        :Parameters:
+          - `ctx`: A mongocrypt_ctx_t. This MongoCryptContext takes ownership
+            of the underlying mongocrypt_ctx_t.
+         - `kms_providers`: The KMS provider map.
+          - `filter`: The filter to use when finding data keys to rewrap in the key vault collection..
+          - `provider`: (optional) The name of a different kms provider.
+          - `master_key`: Optional document for the given provider.
+          - `callback`: A :class:`MongoCryptCallback`.
+        """
+        super(RewrapManyDataKeyContext, self).__init__(ctx, kms_providers)
+        key_encryption_key_bson = None
+        if provider is not None:
+            data = dict(provider=provider)
+            if master_key:
+                data.update(master_key)
+            key_encryption_key_bson = callback.bson_encode(data)
+
+        try:
+            if key_encryption_key_bson:
+                with MongoCryptBinaryIn(key_encryption_key_bson) as binary:
+                    if not lib.mongocrypt_ctx_setopt_key_encryption_key(ctx, binary.bin):
+                        self._raise_from_status()
+
+            filter_bson = callback.bson_encode(filter)
+
+            with MongoCryptBinaryIn(filter_bson) as binary:
+                if not lib.mongocrypt_ctx_rewrap_many_datakey_init(ctx, binary.bin):
+                    self._raise_from_status()
+        except Exception:
+            # Destroy the context on error.
+            self._close()
+            raise
