@@ -14,15 +14,18 @@ set -o xtrace   # Write all commands first to stderr
 set -o errexit  # Exit the script with error if any of the commands fail
 
 # The libmongocrypt git revision release to embed in our wheels.
-REVISION=$(git rev-list -n 1 1.5.0-rc1)
+REVISION=$(git rev-list -n 1 1.7.0-alpha1)
+# The libmongocrypt release branch.
+BRANCH="master"
+MACOS_TARGET=${MACOS_TARGET:="macos"}
 
 if [ "Windows_NT" = "$OS" ]; then # Magic variable in cygwin
     rm -rf venv37
     virtualenv -p C:\\python\\Python37\\python.exe venv37 && . ./venv37/Scripts/activate
 
-    # Build the Windows wheel
+    # Build the Windows wheel.
     rm -rf build libmongocrypt pymongocrypt/*.so pymongocrypt/*.dll pymongocrypt/*.dylib
-    curl -O https://s3.amazonaws.com/mciuploads/libmongocrypt/windows-test/master/${REVISION}/libmongocrypt.tar.gz
+    curl -O https://s3.amazonaws.com/mciuploads/libmongocrypt/windows-test/${BRANCH}/${REVISION}/libmongocrypt.tar.gz
     mkdir libmongocrypt
     tar xzf libmongocrypt.tar.gz -C ./libmongocrypt
     NOCRYPTO_SO=libmongocrypt/nocrypto/bin/mongocrypt.dll
@@ -30,17 +33,41 @@ if [ "Windows_NT" = "$OS" ]; then # Magic variable in cygwin
     cp ${NOCRYPTO_SO} pymongocrypt/
     rm -rf ./libmongocrypt libmongocrypt.tar.gz
 
+    # Ensure updated deps.
+    python -m pip install --upgrade pip setuptools wheel
+
     python setup.py bdist_wheel
     rm -rf build libmongocrypt pymongocrypt/*.so pymongocrypt/*.dll pymongocrypt/*.dylib
     ls dist
 elif [ "Darwin" = "$(uname -s)" ]; then
-    # Build the source dist first
-    rm -rf build pymongocrypt/*.so pymongocrypt/*.dll pymongocrypt/*.dylib
-    python3.7 setup.py sdist
+    # Build the mac wheel.
+    PYTHON=${PYTHON:="/Library/Frameworks/Python.framework/Versions/3.10/bin/python3"}
+
+    # Ensure updated deps.
+    $PYTHON -m pip install --upgrade pip setuptools wheel
 
-    # Build the manylinux2010 wheels
     rm -rf build libmongocrypt pymongocrypt/*.so pymongocrypt/*.dll pymongocrypt/*.dylib
-    curl -O https://s3.amazonaws.com/mciuploads/libmongocrypt/rhel-62-64-bit/master/${REVISION}/libmongocrypt.tar.gz
+
+    # Install the sdist.
+    $PYTHON setup.py sdist
+
+    curl -O https://s3.amazonaws.com/mciuploads/libmongocrypt/${MACOS_TARGET}/${BRANCH}/${REVISION}/libmongocrypt.tar.gz
+    mkdir libmongocrypt
+    tar xzf libmongocrypt.tar.gz -C ./libmongocrypt
+    NOCRYPTO_SO=libmongocrypt/nocrypto/lib/libmongocrypt.dylib
+    chmod +x ${NOCRYPTO_SO}
+    cp ${NOCRYPTO_SO} pymongocrypt/
+    rm -rf ./libmongocrypt libmongocrypt.tar.gz
+
+    $PYTHON setup.py bdist_wheel
+    rm -rf build libmongocrypt pymongocrypt/*.so pymongocrypt/*.dll pymongocrypt/*.dylib
+    ls dist
+fi
+
+if [ $(command -v docker) ]; then
+    # Build the manylinux2010 wheels.
+    rm -rf build libmongocrypt pymongocrypt/*.so pymongocrypt/*.dll pymongocrypt/*.dylib
+    curl -O https://s3.amazonaws.com/mciuploads/libmongocrypt/rhel-62-64-bit/${BRANCH}/${REVISION}/libmongocrypt.tar.gz
     mkdir libmongocrypt
     tar xzf libmongocrypt.tar.gz -C ./libmongocrypt
     NOCRYPTO_SO=libmongocrypt/nocrypto/lib64/libmongocrypt.so
@@ -59,20 +86,7 @@ elif [ "Darwin" = "$(uname -s)" ]; then
         docker run --rm -v `pwd`:/python $image /python/build-manylinux-wheel.sh
     done
 
-    # Build the mac wheel
-    rm -rf build libmongocrypt pymongocrypt/*.so pymongocrypt/*.dll pymongocrypt/*.dylib
-    curl -O https://s3.amazonaws.com/mciuploads/libmongocrypt/macos/master/${REVISION}/libmongocrypt.tar.gz
-    mkdir libmongocrypt
-    tar xzf libmongocrypt.tar.gz -C ./libmongocrypt
-    NOCRYPTO_SO=libmongocrypt/nocrypto/lib/libmongocrypt.dylib
-    chmod +x ${NOCRYPTO_SO}
-    cp ${NOCRYPTO_SO} pymongocrypt/
-    rm -rf ./libmongocrypt libmongocrypt.tar.gz
-
-    python3.7 setup.py bdist_wheel
-    rm -rf build libmongocrypt pymongocrypt/*.so pymongocrypt/*.dll pymongocrypt/*.dylib
+    # Sudo is needed to remove the files created by docker.
+    sudo rm -rf build libmongocrypt pymongocrypt/*.so pymongocrypt/*.dll pymongocrypt/*.dylib
     ls dist
-else
-   echo "ERROR: Run this script on macOS or Windows"
-   exit 1
 fi
