@@ -29,3 +29,9 @@ except ImportError:
 # deprecated assertRaisesRegexp, with a 'p'.
 if not hasattr(unittest.TestCase, 'assertRaisesRegex'):
     unittest.TestCase.assertRaisesRegex = unittest.TestCase.assertRaisesRegexp
+
+try:
+    from unittest import mock
+except ImportError:  # python 2
+    import mock
+
