@@ -16,11 +16,12 @@
 
 import base64
 import copy
+import json
 import os
 import sys
-import uuid
 
 import bson
+from bson.raw_bson import RawBSONDocument
 from bson import json_util
 from bson.binary import Binary, UuidRepresentation
 from bson.codec_options import CodecOptions
@@ -29,6 +30,8 @@ from bson.son import SON
 
 sys.path[0:0] = [""]
 
+import pymongocrypt.mongocrypt
+from pymongo_auth_aws.auth import AwsCredential
 from pymongocrypt.auto_encrypter import AutoEncrypter
 from pymongocrypt.binding import lib
 from pymongocrypt.compat import unicode_type, PY3
@@ -40,7 +43,11 @@ from pymongocrypt.mongocrypt import (MongoCrypt,
                                      MongoCryptOptions)
 from pymongocrypt.state_machine import MongoCryptCallback
 
-from test import unittest
+from test import unittest, mock
+
+import requests.exceptions
+import requests_mock
+
 
 # Data for testing libbmongocrypt binding.
 DATA_DIR = os.path.realpath(os.path.join(os.path.dirname(__file__), 'data'))
@@ -85,6 +92,7 @@ class TestMongoCryptOptions(unittest.TestCase):
         schema_map = bson_data('schema-map.json')
         valid = [
             ({'local': {'key': b'1' * 96}}, None),
+            ({ 'aws' : {} }, schema_map),
             ({'aws': {'accessKeyId': '', 'secretAccessKey': ''}}, schema_map),
             ({'aws': {'accessKeyId': 'foo', 'secretAccessKey': 'foo'}}, None),
             ({'aws': {'accessKeyId': 'foo', 'secretAccessKey': 'foo',
@@ -93,6 +101,9 @@ class TestMongoCryptOptions(unittest.TestCase):
               'local': {'key': b'1' * 96}}, None),
             ({'local': {'key': to_base64(b'1' * 96)}}, None),
             ({'local': {'key': Binary(b'1' * 96)}}, None),
+            ({'azure': {}}, None),
+            ({'azure': {'clientId': 'foo', 'clientSecret': 'bar'}}, None),
+            ({'gcp': {}}, None),
             ({'gcp': {'email': 'foo@bar.baz',
                       'privateKey': b'1'}}, None),
             ({'gcp': {'email': 'foo@bar.baz',
@@ -118,20 +129,19 @@ class TestMongoCryptOptions(unittest.TestCase):
                 ValueError, 'at least one KMS provider must be configured'):
             MongoCryptOptions({})
         for invalid_kms_providers in [
-                {'aws': {}},
                 {'aws': {'accessKeyId': 'foo'}},
                 {'aws': {'secretAccessKey': 'foo'}}]:
             with self.assertRaisesRegex(
-                    ValueError, "kms_providers\['aws'\] must contain "
+                    ValueError, r"kms_providers\['aws'\] must contain "
                                 "'accessKeyId' and 'secretAccessKey'"):
                 MongoCryptOptions(invalid_kms_providers)
         with self.assertRaisesRegex(
-                TypeError, "kms_providers\['local'\]\['key'\] must be an "
-                           "instance of bytes or str \(unicode in Python 2\)"):
+                TypeError, r"kms_providers\['local'\]\['key'\] must be an "
+                           r"instance of bytes or str \(unicode in Python 2\)"):
             MongoCryptOptions({'local': {'key': None}})
         with self.assertRaisesRegex(
-                TypeError, "kms_providers\['gcp'\]\['privateKey'\] must be an "
-                           "instance of bytes or str \(unicode in Python 2\)"):
+                TypeError, r"kms_providers\['gcp'\]\['privateKey'\] must be an "
+                           r"instance of bytes or str \(unicode in Python 2\)"):
             MongoCryptOptions({'gcp': {'email': "foo@bar.baz",
                                        "privateKey": None}})
 
@@ -245,7 +255,7 @@ class TestMongoCrypt(unittest.TestCase):
         with km_contexts[0] as kms_ctx:
             self.assertEqual(kms_ctx.kms_provider, "aws")
             self.assertEqual(kms_ctx.endpoint, "kms.us-east-1.amazonaws.com:443")
-            self.assertEqual(len(kms_ctx.message), 781)
+            self.assertEqual(len(kms_ctx.message), 790)
             self.assertEqual(kms_ctx.bytes_needed, 1024)
 
             kms_ctx.feed(http_data('kms-reply.txt'))
@@ -430,6 +440,175 @@ class TestMongoCryptCallback(unittest.TestCase):
                          json_data('command-reply.json'))
         self.assertEqual(decrypted, bson_data('command-reply.json'))
 
+    def test_need_kms_aws_credentials(self):
+        kms_providers = { 'aws': {} }
+        opts = MongoCryptOptions(kms_providers)
+        callback = MockCallback(
+            list_colls_result=bson_data('collection-info.json'),
+            mongocryptd_reply=bson_data('mongocryptd-reply.json'),
+            key_docs=[bson_data('key-document.json')],
+            kms_reply=http_data('kms-reply.txt'))
+        encrypter = AutoEncrypter(callback, opts)
+        self.addCleanup(encrypter.close)
+
+        with mock.patch("pymongocrypt.credentials.aws_temp_credentials") as m:
+            m.return_value = AwsCredential("example", "example", None)
+            decrypted = encrypter.decrypt(
+                bson_data('encrypted-command-reply.json'))
+            self.assertTrue(m.called)
+
+        self.assertEqual(bson.decode(decrypted, OPTS),
+                         json_data('command-reply.json'))
+        self.assertEqual(decrypted, bson_data('command-reply.json'))
+
+    def test_need_kms_gcp_credentials(self):
+        kms_providers = { 'gcp': {} }
+        opts = MongoCryptOptions(kms_providers)
+        callback = MockCallback(
+            list_colls_result=bson_data('collection-info.json'),
+            mongocryptd_reply=bson_data('mongocryptd-reply.json'),
+            key_docs=[bson_data('key-document-gcp.json')],
+            kms_reply=http_data('kms-reply-gcp.txt'))
+        encrypter = AutoEncrypter(callback, opts)
+        self.addCleanup(encrypter.close)
+
+        with requests_mock.Mocker() as m:
+            data = {"access_token": "foo"}
+            url = "http://metadata.google.internal/computeMetadata/v1/instance/service-accounts/default/token"
+            m.get(url, text=json.dumps(data))
+            decrypted = encrypter.decrypt(
+                bson_data('encrypted-command-reply.json'))
+            self.assertTrue(m.called)
+
+        self.assertEqual(bson.decode(decrypted, OPTS),
+                         json_data('command-reply.json'))
+        self.assertEqual(decrypted, bson_data('command-reply.json'))
+
+
+class TestNeedKMSAzureCredentials(unittest.TestCase):
+    maxDiff = None
+
+    def get_encrypter(self, clear_cache=True):
+        if clear_cache:
+            pymongocrypt.credentials._azure_creds_cache = None
+        kms_providers = { 'azure': {} }
+        opts = MongoCryptOptions(kms_providers)
+        callback = MockCallback(
+            list_colls_result=bson_data('collection-info.json'),
+            mongocryptd_reply=bson_data('mongocryptd-reply.json'),
+            key_docs=[bson_data('key-document-azure.json')],
+            kms_reply=http_data('kms-reply-azure.txt'))
+        encrypter = AutoEncrypter(callback, opts)
+        self.addCleanup(encrypter.close)
+        return encrypter
+
+    def test_success(self):
+        encrypter = self.get_encrypter()
+        with requests_mock.Mocker() as m:
+            data = {"access_token": "foo", "expires_in": 4000}
+            url = "http://169.254.169.254/metadata/identity/oauth2/token"
+            m.get(url, text=json.dumps(data))
+            decrypted = encrypter.decrypt(
+                bson_data('encrypted-command-reply.json'))
+            self.assertTrue(m.called)
+
+        self.assertEqual(bson.decode(decrypted, OPTS),
+                         json_data('command-reply.json'))
+        self.assertEqual(decrypted, bson_data('command-reply.json'))
+        self.assertIsNotNone(pymongocrypt.credentials._azure_creds_cache)
+
+    def test_empty_json(self):
+        encrypter = self.get_encrypter()
+        with requests_mock.Mocker() as m:
+            url = "http://169.254.169.254/metadata/identity/oauth2/token"
+            m.get(url, text=json.dumps({}))
+            with self.assertRaisesRegex(MongoCryptError, "Azure IMDS response must contain"):
+                encrypter.decrypt(bson_data('encrypted-command-reply.json'))
+            self.assertTrue(m.called)
+        self.assertIsNone(pymongocrypt.credentials._azure_creds_cache)
+
+    def test_bad_json(self):
+        encrypter = self.get_encrypter()
+        with requests_mock.Mocker() as m:
+            url = "http://169.254.169.254/metadata/identity/oauth2/token"
+            m.get(url, text="a'")
+            with self.assertRaisesRegex(MongoCryptError, "Azure IMDS response must be in JSON format"):
+                encrypter.decrypt(bson_data('encrypted-command-reply.json'))
+            self.assertTrue(m.called)
+        self.assertIsNone(pymongocrypt.credentials._azure_creds_cache)
+
+    def test_http_404(self):
+        encrypter = self.get_encrypter()
+        with requests_mock.Mocker() as m:
+            url = "http://169.254.169.254/metadata/identity/oauth2/token"
+            m.get(url, status_code=404)
+            with self.assertRaisesRegex(MongoCryptError, "Failed to acquire IMDS access token."):
+                encrypter.decrypt(bson_data('encrypted-command-reply.json'))
+            self.assertTrue(m.called)
+        self.assertIsNone(pymongocrypt.credentials._azure_creds_cache)
+
+    def test_http_500(self):
+        encrypter = self.get_encrypter()
+        with requests_mock.Mocker() as m:
+            url = "http://169.254.169.254/metadata/identity/oauth2/token"
+            m.get(url, status_code=500)
+            with self.assertRaisesRegex(MongoCryptError, "Failed to acquire IMDS access token."):
+                encrypter.decrypt(bson_data('encrypted-command-reply.json'))
+            self.assertTrue(m.called)
+        self.assertIsNone(pymongocrypt.credentials._azure_creds_cache)
+
+    def test_slow_response(self):
+        encrypter = self.get_encrypter()
+        with requests_mock.Mocker() as m:
+            url = "http://169.254.169.254/metadata/identity/oauth2/token"
+            m.get(url, exc=requests.exceptions.ConnectTimeout)
+            with self.assertRaisesRegex(MongoCryptError, "Failed to acquire IMDS access token: "):
+                encrypter.decrypt(bson_data('encrypted-command-reply.json'))
+            self.assertTrue(m.called)
+        self.assertIsNone(pymongocrypt.credentials._azure_creds_cache)
+
+    def test_cache(self):
+        encrypter = self.get_encrypter()
+        with requests_mock.Mocker() as m:
+            data = {"access_token": "foo", "expires_in": 4000}
+            url = "http://169.254.169.254/metadata/identity/oauth2/token"
+            m.get(url, text=json.dumps(data))
+            decrypted = encrypter.decrypt(
+                bson_data('encrypted-command-reply.json'))
+            self.assertTrue(m.called)
+
+        self.assertIsNotNone(pymongocrypt.credentials._azure_creds_cache)
+
+        # Should use the cached value.
+        decrypted = encrypter.decrypt(bson_data('encrypted-command-reply.json'))
+        self.assertEqual(decrypted, bson_data('command-reply.json'))
+
+        self.assertIsNotNone(pymongocrypt.credentials._azure_creds_cache)
+
+    def test_cache_expires_soon(self):
+        encrypter = self.get_encrypter()
+        with requests_mock.Mocker() as m:
+            data = {"access_token": "foo", "expires_in": 10}
+            url = "http://169.254.169.254/metadata/identity/oauth2/token"
+            m.get(url, text=json.dumps(data))
+            decrypted = encrypter.decrypt(
+                bson_data('encrypted-command-reply.json'))
+            self.assertTrue(m.called)
+
+        self.assertIsNotNone(pymongocrypt.credentials._azure_creds_cache)
+
+        # Should not use the cached value.
+        encrypter = self.get_encrypter(False)
+        self.assertIsNotNone(pymongocrypt.credentials._azure_creds_cache)
+        with requests_mock.Mocker() as m:
+            url = "http://169.254.169.254/metadata/identity/oauth2/token"
+            m.get(url, exc=requests.exceptions.ConnectTimeout)
+            with self.assertRaisesRegex(MongoCryptError, "Failed to acquire IMDS access token: "):
+                encrypter.decrypt(bson_data('encrypted-command-reply.json'))
+            self.assertTrue(m.called)
+
+        self.assertIsNone(pymongocrypt.credentials._azure_creds_cache)
+
 
 class KeyVaultCallback(MockCallback):
     def __init__(self, kms_reply=None):
@@ -490,11 +669,6 @@ class TestExplicitEncryption(unittest.TestCase):
         # Invalid algorithm.
         with self.assertRaisesRegex(MongoCryptError, "algorithm"):
             encrypter.encrypt(encoded_val, "Invalid", key_id)
-        # Invalid index_key_id type.
-        with self.assertRaises(TypeError):
-            encrypter.encrypt(encoded_val, "Indexed", key_id, index_key_id=1)
-        with self.assertRaises(MongoCryptError):
-            encrypter.encrypt(encoded_val, "Indexed", key_id, index_key_id=b'1234')
         # Invalid query_type type.
         with self.assertRaisesRegex(TypeError, "query_type"):
             encrypter.encrypt(encoded_val, "Indexed", key_id, query_type=42)
@@ -515,11 +689,9 @@ class TestExplicitEncryption(unittest.TestCase):
 
     def test_encrypt_indexed(self):
         key_path = 'keys/ABCDEFAB123498761234123456789012-local-document.json'
-        index_key_path = 'keys/12345678123498761234123456789012-local-document.json'
         key_id = json_data(key_path)['_id']
-        index_key_id = json_data(index_key_path)['_id']
         encrypter = ExplicitEncrypter(MockCallback(
-            key_docs=[bson_data(key_path), bson_data(index_key_path)],
+            key_docs=[bson_data(key_path)],
             kms_reply=http_data('kms-reply.txt')), self.mongo_crypt_opts())
         self.addCleanup(encrypter.close)
 
@@ -532,7 +704,6 @@ class TestExplicitEncryption(unittest.TestCase):
             dict(algorithm='Unindexed'),
         ]:
             kwargs['key_id'] = key_id
-            kwargs['index_key_id'] = index_key_id
             encrypted = encrypter.encrypt(encoded_val, **kwargs)
             encrypted_val = bson.decode(encrypted, OPTS)['v']
             self.assertIsInstance(encrypted_val, Binary)
@@ -576,9 +747,50 @@ class TestExplicitEncryption(unittest.TestCase):
             "key": "key",
             "endpoint": "example.com"
         }
-        encrypter.create_data_key("aws", master_key=master_key)
+        key_material = base64.b64decode('xPTAjBRG5JiPm+d3fj6XLi2q5DMXUS/f1f+SMAlhhwkhDRL0kr8r9GDLIGTAGlvC+HVjSIgdL+RKwZCvpXSyxTICWSXTUYsWYPyu3IoHbuBZdmw2faM3WhcRIgbMReU5')
+        if not PY3:
+            key_material = Binary(key_material)
+        encrypter.create_data_key("aws", master_key=master_key, key_material=key_material)
         self.assertEqual("example.com:443", mock_key_vault.kms_endpoint)
 
+    def test_data_key_creation_bad_key_material(self):
+        mock_key_vault = KeyVaultCallback(
+            kms_reply=http_data('kms-encrypt-reply.txt'))
+        encrypter = ExplicitEncrypter(mock_key_vault, self.mongo_crypt_opts())
+        self.addCleanup(encrypter.close)
+
+        key_material = Binary(b'0' * 97)
+        with self.assertRaisesRegex(MongoCryptError, "keyMaterial should have length 96, but has length 97"):
+            encrypter.create_data_key("local", key_material=key_material)
+
+    def test_rewrap_many_data_key(self):
+        key_path = 'keys/ABCDEFAB123498761234123456789012-local-document.json'
+        key_path2 = 'keys/12345678123498761234123456789012-local-document.json'
+        encrypter = ExplicitEncrypter(MockCallback(
+            key_docs=[bson_data(key_path), bson_data(key_path2)]), self.mongo_crypt_opts())
+        self.addCleanup(encrypter.close)
+
+        result = encrypter.rewrap_many_data_key({})
+        raw_doc = RawBSONDocument(result)
+        assert len(raw_doc['v']) == 2
+
+    def test_range_query_int32(self):
+        key_path = 'keys/ABCDEFAB123498761234123456789012-local-document.json'
+        key_id = json_data(key_path)['_id']
+        encrypter = ExplicitEncrypter(MockCallback(
+            key_docs=[bson_data(key_path)],
+            kms_reply=http_data('kms-reply.txt')), self.mongo_crypt_opts())
+        self.addCleanup(encrypter.close)
+
+        range_opts = bson_data("fle2-find-range-explicit/int32/rangeopts.json")
+        value = bson_data("fle2-find-range-explicit/int32/value-to-encrypt.json")
+        expected = json_data("fle2-find-range-explicit/int32/encrypted-payload.json")
+        encrypted = encrypter.encrypt(
+            value, "rangePreview", key_id=key_id, query_type="rangePreview",
+            contention_factor=4, range_opts=range_opts, is_expression=True)
+        encrypted_val = bson.decode(encrypted, OPTS)
+        self.assertEqual(encrypted_val, expected)
+
 
 def read(filename, **kwargs):
     with open(os.path.join(DATA_DIR, filename), **kwargs) as fp:
