@@ -300,6 +300,8 @@ class Distro(object):
                 return "bionic"
             elif build_os == 'ubuntu2004':
                 return "focal"
+            elif build_os == 'ubuntu2204':
+                return "jammy"
             else:
                 raise Exception("unsupported build_os: %s" % build_os)
         elif self.dname == 'debian':
@@ -309,6 +311,8 @@ class Distro(object):
                 return 'stretch'
             elif build_os == 'debian10':
                 return 'buster'
+            elif build_os == 'debian11':
+                return 'bullseye'
             else:
                 raise Exception("unsupported build_os: %s" % build_os)
         else:
@@ -347,9 +351,10 @@ class Distro(object):
                 "ubuntu1604",
                 "ubuntu1804",
                 "ubuntu2004",
+                "ubuntu2204",
             ]
         elif self.dname == 'debian':
-            return ["debian81", "debian92", "debian10"]
+            return ["debian81", "debian92", "debian10", "debian11"]
         else:
             raise Exception("BUG: unsupported platform?")
 
