@@ -23,34 +23,88 @@
 #include <CommonCrypto/CommonHMAC.h>
 #include <CommonCrypto/CommonRandom.h>
 
+static const char *
+CCCryptorStatus_to_string (CCCryptorStatus status)
+{
+   switch (status) {
+   case kCCSuccess:
+      return "Success";
+   case kCCParamError:
+      return "ParamError";
+   case kCCBufferTooSmall:
+      return "BufferTooSmall";
+   case kCCMemoryFailure:
+      return "MemoryFailure";
+   case kCCAlignmentError:
+      return "AlignmentError";
+   case kCCDecodeError:
+      return "DecodeError";
+   case kCCUnimplemented:
+      return "Unimplemented";
+   case kCCOverflow:
+      return "Overflow";
+   case kCCRNGFailure:
+      return "RNGFailure";
+   case kCCUnspecifiedError:
+      return "UnspecifiedError";
+   case kCCCallSequenceError:
+      return "CallSequenceError";
+   case kCCKeySizeError:
+      return "KeySizeError";
+   case kCCInvalidKey:
+      return "InvalidKey";
+   default:
+      return "Unknown";
+   }
+}
+
 bool _native_crypto_initialized = false;
 
 void
-_native_crypto_init ()
+_native_crypto_init (void)
 {
    _native_crypto_initialized = true;
 }
 
 
-bool
-_native_crypto_aes_256_cbc_encrypt (aes_256_args_t args)
+static bool
+_native_crypto_aes_256_cbc_encrypt_with_mode (aes_256_args_t args, CCMode mode)
 {
+   BSON_ASSERT (args.iv);
+   BSON_ASSERT (args.key);
+   BSON_ASSERT (args.in);
+   BSON_ASSERT (args.out);
+
    bool ret = false;
    CCCryptorRef ctx = NULL;
    CCCryptorStatus cc_status;
    size_t intermediate_bytes_written;
    mongocrypt_status_t *status = args.status;
 
-   cc_status = CCCryptorCreate (kCCEncrypt,
-                                kCCAlgorithmAES,
-                                0 /* defaults to CBC w/ no padding */,
-                                args.key->data,
-                                kCCKeySizeAES256,
-                                args.iv->data,
-                                &ctx);
+   cc_status = CCCryptorCreateWithMode (kCCEncrypt,
+                                        mode,
+                                        kCCAlgorithmAES,
+                                        0 /* defaults to CBC w/ no padding */,
+                                        args.iv->data,
+                                        args.key->data,
+                                        kCCKeySizeAES256,
+                                        NULL,
+                                        0,
+                                        0,
+                                        0,
+                                        &ctx);
 
    if (cc_status != kCCSuccess) {
-      CLIENT_ERR ("error initializing cipher: %d", (int) cc_status);
+      if (cc_status == kCCUnimplemented && mode == kCCModeCTR) {
+         CLIENT_ERR ("error initializing cipher: %s (%d). CTR mode is only "
+                     "supported on macOS 10.15+",
+                     CCCryptorStatus_to_string (cc_status),
+                     (int) cc_status);
+      } else {
+         CLIENT_ERR ("error initializing cipher: %s (%d)",
+                     CCCryptorStatus_to_string (cc_status),
+                     (int) cc_status);
+      }
       goto done;
    }
 
@@ -63,20 +117,26 @@ _native_crypto_aes_256_cbc_encrypt (aes_256_args_t args)
                                 args.out->len,
                                 &intermediate_bytes_written);
    if (cc_status != kCCSuccess) {
-      CLIENT_ERR ("error encrypting: %d", (int) cc_status);
+      CLIENT_ERR ("error encrypting: %s (%d)",
+                  CCCryptorStatus_to_string (cc_status),
+                  (int) cc_status);
       goto done;
    }
-   *args.bytes_written = intermediate_bytes_written;
-
+   BSON_ASSERT (intermediate_bytes_written <= UINT32_MAX);
+   *args.bytes_written = (uint32_t) intermediate_bytes_written;
 
+   BSON_ASSERT (args.out->len >= *args.bytes_written);
    cc_status = CCCryptorFinal (ctx,
                                args.out->data + *args.bytes_written,
                                args.out->len - *args.bytes_written,
                                &intermediate_bytes_written);
+   BSON_ASSERT (UINT32_MAX - *args.bytes_written >= intermediate_bytes_written);
    *args.bytes_written += intermediate_bytes_written;
 
    if (cc_status != kCCSuccess) {
-      CLIENT_ERR ("error finalizing: %d", (int) cc_status);
+      CLIENT_ERR ("error finalizing: %s (%d)",
+                  CCCryptorStatus_to_string (cc_status),
+                  (int) cc_status);
       goto done;
    }
 
@@ -86,28 +146,58 @@ done:
    return ret;
 }
 
+bool
+_native_crypto_aes_256_cbc_encrypt (aes_256_args_t args)
+{
+   return _native_crypto_aes_256_cbc_encrypt_with_mode (args, kCCModeCBC);
+}
+
+bool
+_native_crypto_aes_256_ctr_encrypt (aes_256_args_t args)
+{
+   return _native_crypto_aes_256_cbc_encrypt_with_mode (args, kCCModeCTR);
+}
 
 /* Note, the decrypt function is almost exactly the same as the encrypt
  * functions except for the kCCDecrypt and the error message. */
-bool
-_native_crypto_aes_256_cbc_decrypt (aes_256_args_t args)
+static bool
+_native_crypto_aes_256_cbc_decrypt_with_mode (aes_256_args_t args, CCMode mode)
 {
+   BSON_ASSERT (args.iv);
+   BSON_ASSERT (args.key);
+   BSON_ASSERT (args.in);
+   BSON_ASSERT (args.out);
+
    bool ret = false;
    CCCryptorRef ctx = NULL;
    CCCryptorStatus cc_status;
    size_t intermediate_bytes_written;
    mongocrypt_status_t *status = args.status;
 
-   cc_status = CCCryptorCreate (kCCDecrypt,
-                                kCCAlgorithmAES,
-                                0 /* defaults to CBC w/ no padding */,
-                                args.key->data,
-                                kCCKeySizeAES256,
-                                args.iv->data,
-                                &ctx);
+   cc_status = CCCryptorCreateWithMode (kCCDecrypt,
+                                        mode,
+                                        kCCAlgorithmAES,
+                                        0 /* defaults to CBC w/ no padding */,
+                                        args.iv->data,
+                                        args.key->data,
+                                        kCCKeySizeAES256,
+                                        NULL,
+                                        0,
+                                        0,
+                                        0,
+                                        &ctx);
 
    if (cc_status != kCCSuccess) {
-      CLIENT_ERR ("error initializing cipher: %d", (int) cc_status);
+      if (cc_status == kCCUnimplemented && mode == kCCModeCTR) {
+         CLIENT_ERR ("error initializing cipher: %s (%d). CTR mode is only "
+                     "supported on macOS 10.15+",
+                     CCCryptorStatus_to_string (cc_status),
+                     (int) cc_status);
+      } else {
+         CLIENT_ERR ("error initializing cipher: %s (%d)",
+                     CCCryptorStatus_to_string (cc_status),
+                     (int) cc_status);
+      }
       goto done;
    }
 
@@ -119,19 +209,26 @@ _native_crypto_aes_256_cbc_decrypt (aes_256_args_t args)
                                 args.out->len,
                                 &intermediate_bytes_written);
    if (cc_status != kCCSuccess) {
-      CLIENT_ERR ("error decrypting: %d", (int) cc_status);
+      CLIENT_ERR ("error decrypting: %s (%d)",
+                  CCCryptorStatus_to_string (cc_status),
+                  (int) cc_status);
       goto done;
    }
-   *args.bytes_written = intermediate_bytes_written;
+   BSON_ASSERT (intermediate_bytes_written <= UINT32_MAX);
+   *args.bytes_written = (uint32_t) intermediate_bytes_written;
 
+   BSON_ASSERT (args.out->len >= *args.bytes_written);
    cc_status = CCCryptorFinal (ctx,
                                args.out->data + *args.bytes_written,
                                args.out->len - *args.bytes_written,
                                &intermediate_bytes_written);
+   BSON_ASSERT (UINT32_MAX - *args.bytes_written >= intermediate_bytes_written);
    *args.bytes_written += intermediate_bytes_written;
 
    if (cc_status != kCCSuccess) {
-      CLIENT_ERR ("error finalizing: %d", (int) cc_status);
+      CLIENT_ERR ("error finalizing: %s (%d)",
+                  CCCryptorStatus_to_string (cc_status),
+                  (int) cc_status);
       goto done;
    }
 
@@ -141,6 +238,18 @@ done:
    return ret;
 }
 
+bool
+_native_crypto_aes_256_cbc_decrypt (aes_256_args_t args)
+{
+   return _native_crypto_aes_256_cbc_decrypt_with_mode (args, kCCModeCBC);
+}
+
+bool
+_native_crypto_aes_256_ctr_decrypt (aes_256_args_t args)
+{
+   return _native_crypto_aes_256_cbc_decrypt_with_mode (args, kCCModeCTR);
+}
+
 
 /* _hmac_with_algorithm computes an HMAC of @in with the algorithm specified by
  * @algorithm.
@@ -148,7 +257,7 @@ done:
  * @out is the output. @out must be allocated by the caller with
  * the expected length @expect_out_len for the output.
  * Returns false and sets @status on error. @status is required. */
-bool
+static bool
 _hmac_with_algorithm (CCHmacAlgorithm algorithm,
                       const _mongocrypt_buffer_t *key,
                       const _mongocrypt_buffer_t *in,
@@ -158,6 +267,10 @@ _hmac_with_algorithm (CCHmacAlgorithm algorithm,
 {
    CCHmacContext *ctx;
 
+   BSON_ASSERT_PARAM (key);
+   BSON_ASSERT_PARAM (in);
+   BSON_ASSERT_PARAM (out);
+
    if (out->len != expect_out_len) {
       CLIENT_ERR ("out does not contain %" PRIu32 " bytes", expect_out_len);
       return false;
@@ -166,7 +279,7 @@ _hmac_with_algorithm (CCHmacAlgorithm algorithm,
    ctx = bson_malloc0 (sizeof (*ctx));
    BSON_ASSERT (ctx);
 
-
+   /* The ->len members are uint32_t and these functions take size_t */
    CCHmacInit (ctx, algorithm, key->data, key->len);
    CCHmacUpdate (ctx, in->data, in->len);
    CCHmacFinal (ctx, out->data);
@@ -190,6 +303,8 @@ _native_crypto_random (_mongocrypt_buffer_t *out,
                        uint32_t count,
                        mongocrypt_status_t *status)
 {
+   BSON_ASSERT_PARAM (out);
+
    CCRNGStatus ret = CCRandomGenerateBytes (out->data, (size_t) count);
    if (ret != kCCSuccess) {
       CLIENT_ERR ("failed to generate random iv: %d", (int) ret);
@@ -198,24 +313,6 @@ _native_crypto_random (_mongocrypt_buffer_t *out,
    return true;
 }
 
-bool
-_native_crypto_aes_256_ctr_encrypt (aes_256_args_t args)
-{
-   mongocrypt_status_t *status = args.status;
-   CLIENT_ERR (
-      "_native_crypto_aes_256_ctr_encrypt not implemented for CommonCrypto");
-   return false;
-}
-
-bool
-_native_crypto_aes_256_ctr_decrypt (aes_256_args_t args)
-{
-   mongocrypt_status_t *status = args.status;
-   CLIENT_ERR (
-      "_native_crypto_aes_256_ctr_decrypt not implemented for CommonCrypto");
-   return false;
-}
-
 bool
 _native_crypto_hmac_sha_256 (const _mongocrypt_buffer_t *key,
                              const _mongocrypt_buffer_t *in,
