@@ -24,6 +24,8 @@ _cleanup (mongocrypt_ctx_t *ctx)
 {
    _mongocrypt_ctx_datakey_t *dkctx;
 
+   BSON_ASSERT_PARAM (ctx);
+
    dkctx = (_mongocrypt_ctx_datakey_t *) ctx;
    _mongocrypt_buffer_cleanup (&dkctx->key_doc);
    _mongocrypt_kms_ctx_cleanup (&dkctx->kms);
@@ -38,6 +40,8 @@ _next_kms_ctx (mongocrypt_ctx_t *ctx)
 {
    _mongocrypt_ctx_datakey_t *dkctx;
 
+   BSON_ASSERT_PARAM (ctx);
+
    dkctx = (_mongocrypt_ctx_datakey_t *) ctx;
    if (dkctx->kms_returned) {
       return NULL;
@@ -56,6 +60,8 @@ _kms_kmip_start (mongocrypt_ctx_t *ctx)
    mongocrypt_status_t *status = ctx->status;
    _mongocrypt_buffer_t secretdata = {0};
 
+   BSON_ASSERT_PARAM (ctx);
+
    if (ctx->opts.kek.kms_provider != MONGOCRYPT_KMS_PROVIDER_KMIP) {
       CLIENT_ERR ("KMS provider is not KMIP");
       goto fail;
@@ -65,8 +71,8 @@ _kms_kmip_start (mongocrypt_ctx_t *ctx)
 
    if (ctx->opts.kek.provider.kmip.endpoint) {
       endpoint = ctx->opts.kek.provider.kmip.endpoint;
-   } else if (_mongocrypt_ctx_kms_providers(ctx)->kmip.endpoint) {
-      endpoint = _mongocrypt_ctx_kms_providers(ctx)->kmip.endpoint;
+   } else if (_mongocrypt_ctx_kms_providers (ctx)->kmip.endpoint) {
+      endpoint = _mongocrypt_ctx_kms_providers (ctx)->kmip.endpoint;
    } else {
       CLIENT_ERR ("endpoint not set for KMIP request");
       goto fail;
@@ -178,11 +184,13 @@ fail:
 static bool
 _kms_start (mongocrypt_ctx_t *ctx)
 {
+   BSON_ASSERT_PARAM (ctx);
+
    bool ret = false;
    _mongocrypt_ctx_datakey_t *dkctx;
    char *access_token = NULL;
    _mongocrypt_opts_kms_providers_t *const kms_providers =
-      _mongocrypt_ctx_kms_providers(ctx);
+      _mongocrypt_ctx_kms_providers (ctx);
 
    dkctx = (_mongocrypt_ctx_datakey_t *) ctx;
 
@@ -218,8 +226,12 @@ _kms_start (mongocrypt_ctx_t *ctx)
 
       ctx->state = MONGOCRYPT_CTX_NEED_KMS;
    } else if (ctx->opts.kek.kms_provider == MONGOCRYPT_KMS_PROVIDER_AZURE) {
-      access_token =
-         _mongocrypt_cache_oauth_get (ctx->crypt->cache_oauth_azure);
+      if (ctx->kms_providers.azure.access_token) {
+         access_token = bson_strdup (ctx->kms_providers.azure.access_token);
+      } else {
+         access_token =
+            _mongocrypt_cache_oauth_get (ctx->crypt->cache_oauth_azure);
+      }
       if (access_token) {
          if (!_mongocrypt_kms_ctx_init_azure_wrapkey (
                 &dkctx->kms,
@@ -245,7 +257,13 @@ _kms_start (mongocrypt_ctx_t *ctx)
       }
       ctx->state = MONGOCRYPT_CTX_NEED_KMS;
    } else if (ctx->opts.kek.kms_provider == MONGOCRYPT_KMS_PROVIDER_GCP) {
-      access_token = _mongocrypt_cache_oauth_get (ctx->crypt->cache_oauth_gcp);
+      if (NULL != ctx->kms_providers.gcp.access_token) {
+         access_token =
+            bson_strdup ((const char *) ctx->kms_providers.gcp.access_token);
+      } else {
+         access_token =
+            _mongocrypt_cache_oauth_get (ctx->crypt->cache_oauth_gcp);
+      }
       if (access_token) {
          if (!_mongocrypt_kms_ctx_init_gcp_encrypt (
                 &dkctx->kms,
@@ -292,6 +310,8 @@ _kms_done (mongocrypt_ctx_t *ctx)
    _mongocrypt_ctx_datakey_t *dkctx;
    mongocrypt_status_t *status;
 
+   BSON_ASSERT_PARAM (ctx);
+
    dkctx = (_mongocrypt_ctx_datakey_t *) ctx;
    status = ctx->status;
    if (!mongocrypt_kms_ctx_status (&dkctx->kms, ctx->status)) {
@@ -358,7 +378,9 @@ static bool
 _append_id (mongocrypt_t *crypt, bson_t *bson, mongocrypt_status_t *status)
 {
    _mongocrypt_buffer_t uuid;
-#define UUID_LEN 16
+
+   BSON_ASSERT_PARAM (crypt);
+   BSON_ASSERT_PARAM (bson);
 
    _mongocrypt_buffer_init (&uuid);
    uuid.data = bson_malloc (UUID_LEN);
@@ -392,6 +414,9 @@ _finalize (mongocrypt_ctx_t *ctx, mongocrypt_binary_t *out)
    bson_t key_doc, child;
    struct timeval tp;
 
+   BSON_ASSERT_PARAM (ctx);
+   BSON_ASSERT_PARAM (out);
+
 #define BSON_CHECK(_stmt)                                                      \
    if (!(_stmt)) {                                                             \
       bson_destroy (&key_doc);                                                 \
