@@ -19,14 +19,12 @@
 #include "mongocrypt-ctx-private.h"
 #include "mongocrypt-key-broker-private.h"
 
-#define ALGORITHM_DETERMINISTIC "AEAD_AES_256_CBC_HMAC_SHA_512-Deterministic"
-#define ALGORITHM_DETERMINISTIC_LEN 43
-#define ALGORITHM_RANDOM "AEAD_AES_256_CBC_HMAC_SHA_512-Random"
-#define ALGORITHM_RANDOM_LEN 36
-
 bool
 _mongocrypt_ctx_fail_w_msg (mongocrypt_ctx_t *ctx, const char *msg)
 {
+   BSON_ASSERT_PARAM (ctx);
+   BSON_ASSERT_PARAM (msg);
+
    _mongocrypt_set_error (ctx->status,
                           MONGOCRYPT_STATUS_ERROR_CLIENT,
                           MONGOCRYPT_GENERIC_ERROR_CODE,
@@ -39,6 +37,8 @@ _mongocrypt_ctx_fail_w_msg (mongocrypt_ctx_t *ctx, const char *msg)
 bool
 _mongocrypt_ctx_fail (mongocrypt_ctx_t *ctx)
 {
+   BSON_ASSERT_PARAM (ctx);
+
    if (mongocrypt_status_ok (ctx->status)) {
       return _mongocrypt_ctx_fail_w_msg (
          ctx, "unexpected, failing but no error status set");
@@ -54,7 +54,7 @@ _set_binary_opt (mongocrypt_ctx_t *ctx,
                  _mongocrypt_buffer_t *buf,
                  bson_subtype_t subtype)
 {
-   BSON_ASSERT (ctx);
+   BSON_ASSERT_PARAM (ctx);
 
    if (ctx->state == MONGOCRYPT_CTX_ERROR) {
       return false;
@@ -93,8 +93,10 @@ mongocrypt_ctx_setopt_key_id (mongocrypt_ctx_t *ctx,
 
    if (ctx->crypt->log.trace_enabled && key_id && key_id->data) {
       char *key_id_val;
+      /* this should never happen, so assert rather than return false */
+      BSON_ASSERT (key_id->len <= INT_MAX);
       key_id_val =
-         _mongocrypt_new_string_from_bytes (key_id->data, key_id->len);
+         _mongocrypt_new_string_from_bytes (key_id->data, (int) key_id->len);
       _mongocrypt_log (&ctx->crypt->log,
                        MONGOCRYPT_LOG_LEVEL_TRACE,
                        "%s (%s=\"%s\")",
@@ -251,8 +253,6 @@ mongocrypt_ctx_setopt_algorithm (mongocrypt_ctx_t *ctx,
                                  const char *algorithm,
                                  int len)
 {
-   size_t calculated_len;
-
    if (!ctx) {
       return false;
    }
@@ -265,7 +265,8 @@ mongocrypt_ctx_setopt_algorithm (mongocrypt_ctx_t *ctx,
       return false;
    }
 
-   if (ctx->opts.algorithm != MONGOCRYPT_ENCRYPTION_ALGORITHM_NONE) {
+   if (ctx->opts.algorithm != MONGOCRYPT_ENCRYPTION_ALGORITHM_NONE ||
+       ctx->opts.index_type.set) {
       return _mongocrypt_ctx_fail_w_msg (ctx, "already set algorithm");
    }
 
@@ -277,32 +278,48 @@ mongocrypt_ctx_setopt_algorithm (mongocrypt_ctx_t *ctx,
       return _mongocrypt_ctx_fail_w_msg (ctx, "passed null algorithm");
    }
 
-   calculated_len = len == -1 ? strlen (algorithm) : (size_t) len;
+   const size_t calculated_len = len == -1 ? strlen (algorithm) : (size_t) len;
    if (ctx->crypt->log.trace_enabled) {
       _mongocrypt_log (&ctx->crypt->log,
                        MONGOCRYPT_LOG_LEVEL_TRACE,
                        "%s (%s=\"%.*s\")",
                        BSON_FUNC,
                        "algorithm",
-                       (int) calculated_len,
+                       calculated_len <= (size_t) INT_MAX ? (int) calculated_len
+                                                          : INT_MAX,
                        algorithm);
    }
 
-   if (calculated_len == ALGORITHM_DETERMINISTIC_LEN &&
-       strncmp (algorithm,
-                ALGORITHM_DETERMINISTIC,
-                ALGORITHM_DETERMINISTIC_LEN) == 0) {
+   mstr_view algo_str = mstrv_view_data (algorithm, calculated_len);
+   if (mstr_eq_ignore_case (
+          algo_str, mstrv_lit (MONGOCRYPT_ALGORITHM_DETERMINISTIC_STR))) {
       ctx->opts.algorithm = MONGOCRYPT_ENCRYPTION_ALGORITHM_DETERMINISTIC;
-      return true;
-   }
-
-   if (calculated_len == ALGORITHM_RANDOM_LEN &&
-       strncmp (algorithm, ALGORITHM_RANDOM, ALGORITHM_RANDOM_LEN) == 0) {
+   } else if (mstr_eq_ignore_case (
+                 algo_str, mstrv_lit (MONGOCRYPT_ALGORITHM_RANDOM_STR))) {
       ctx->opts.algorithm = MONGOCRYPT_ENCRYPTION_ALGORITHM_RANDOM;
-      return true;
+   } else if (mstr_eq_ignore_case (
+                 algo_str, mstrv_lit (MONGOCRYPT_ALGORITHM_INDEXED_STR))) {
+      ctx->opts.index_type.value = MONGOCRYPT_INDEX_TYPE_EQUALITY;
+      ctx->opts.index_type.set = true;
+   } else if (mstr_eq_ignore_case (
+                 algo_str, mstrv_lit (MONGOCRYPT_ALGORITHM_UNINDEXED_STR))) {
+      ctx->opts.index_type.value = MONGOCRYPT_INDEX_TYPE_NONE;
+      ctx->opts.index_type.set = true;
+   } else if (mstr_eq_ignore_case (
+                 algo_str, mstrv_lit (MONGOCRYPT_ALGORITHM_RANGEPREVIEW_STR))) {
+      ctx->opts.index_type.value = MONGOCRYPT_INDEX_TYPE_RANGEPREVIEW;
+      ctx->opts.index_type.set = true;
+   } else {
+      char *error = bson_strdup_printf (
+         "unsupported algorithm string \"%.*s\"",
+         algo_str.len <= (size_t) INT_MAX ? (int) algo_str.len : INT_MAX,
+         algo_str.data);
+      _mongocrypt_ctx_fail_w_msg (ctx, error);
+      bson_free (error);
+      return false;
    }
 
-   return _mongocrypt_ctx_fail_w_msg (ctx, "unsupported algorithm");
+   return true;
 }
 
 
@@ -351,6 +368,9 @@ mongocrypt_ctx_new (mongocrypt_t *crypt)
 static bool
 _mongo_op_keys (mongocrypt_ctx_t *ctx, mongocrypt_binary_t *out)
 {
+   BSON_ASSERT_PARAM (ctx);
+   BSON_ASSERT_PARAM (out);
+
    /* Construct the find filter to fetch keys. */
    if (!_mongocrypt_key_broker_filter (&ctx->kb, out)) {
       BSON_ASSERT (!_mongocrypt_key_broker_status (&ctx->kb, ctx->status));
@@ -365,11 +385,12 @@ _mongo_feed_keys (mongocrypt_ctx_t *ctx, mongocrypt_binary_t *in)
 {
    _mongocrypt_buffer_t buf;
 
+   BSON_ASSERT_PARAM (ctx);
+   BSON_ASSERT_PARAM (in);
+
    _mongocrypt_buffer_from_binary (&buf, in);
    if (!_mongocrypt_key_broker_add_doc (
-         &ctx->kb,
-         _mongocrypt_ctx_kms_providers(ctx),
-         &buf)) {
+          &ctx->kb, _mongocrypt_ctx_kms_providers (ctx), &buf)) {
       BSON_ASSERT (!_mongocrypt_key_broker_status (&ctx->kb, ctx->status));
       return _mongocrypt_ctx_fail (ctx);
    }
@@ -380,6 +401,8 @@ _mongo_feed_keys (mongocrypt_ctx_t *ctx, mongocrypt_binary_t *in)
 static bool
 _mongo_done_keys (mongocrypt_ctx_t *ctx)
 {
+   BSON_ASSERT_PARAM (ctx);
+
    (void) _mongocrypt_key_broker_docs_done (&ctx->kb);
    return _mongocrypt_ctx_state_from_key_broker (ctx);
 }
@@ -387,6 +410,8 @@ _mongo_done_keys (mongocrypt_ctx_t *ctx)
 static mongocrypt_kms_ctx_t *
 _next_kms_ctx (mongocrypt_ctx_t *ctx)
 {
+   BSON_ASSERT_PARAM (ctx);
+
    return _mongocrypt_key_broker_next_kms (&ctx->kb);
 }
 
@@ -394,8 +419,12 @@ _next_kms_ctx (mongocrypt_ctx_t *ctx)
 static bool
 _kms_done (mongocrypt_ctx_t *ctx)
 {
-   _mongocrypt_opts_kms_providers_t* kms_providers =
-      _mongocrypt_ctx_kms_providers(ctx);
+   _mongocrypt_opts_kms_providers_t *kms_providers;
+
+   BSON_ASSERT_PARAM (ctx);
+
+   kms_providers = _mongocrypt_ctx_kms_providers (ctx);
+
    if (!_mongocrypt_key_broker_kms_done (&ctx->kb, kms_providers)) {
       BSON_ASSERT (!_mongocrypt_key_broker_status (&ctx->kb, ctx->status));
       return _mongocrypt_ctx_fail (ctx);
@@ -415,7 +444,7 @@ mongocrypt_ctx_mongo_op (mongocrypt_ctx_t *ctx, mongocrypt_binary_t *out)
    }
 
    if (!out) {
-      return _mongocrypt_ctx_fail_w_msg (ctx, "invalid NULL input");
+      return _mongocrypt_ctx_fail_w_msg (ctx, "invalid NULL output");
    }
 
    switch (ctx->state) {
@@ -427,6 +456,10 @@ mongocrypt_ctx_mongo_op (mongocrypt_ctx_t *ctx, mongocrypt_binary_t *out)
       CHECK_AND_CALL (mongo_op_keys, ctx, out);
    case MONGOCRYPT_CTX_ERROR:
       return false;
+   case MONGOCRYPT_CTX_DONE:
+   case MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS:
+   case MONGOCRYPT_CTX_NEED_KMS:
+   case MONGOCRYPT_CTX_READY:
    default:
       return _mongocrypt_ctx_fail_w_msg (ctx, "wrong state");
    }
@@ -469,6 +502,10 @@ mongocrypt_ctx_mongo_feed (mongocrypt_ctx_t *ctx, mongocrypt_binary_t *in)
       CHECK_AND_CALL (mongo_feed_keys, ctx, in);
    case MONGOCRYPT_CTX_ERROR:
       return false;
+   case MONGOCRYPT_CTX_DONE:
+   case MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS:
+   case MONGOCRYPT_CTX_NEED_KMS:
+   case MONGOCRYPT_CTX_READY:
    default:
       return _mongocrypt_ctx_fail_w_msg (ctx, "wrong state");
    }
@@ -494,6 +531,10 @@ mongocrypt_ctx_mongo_done (mongocrypt_ctx_t *ctx)
       CHECK_AND_CALL (mongo_done_keys, ctx);
    case MONGOCRYPT_CTX_ERROR:
       return false;
+   case MONGOCRYPT_CTX_DONE:
+   case MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS:
+   case MONGOCRYPT_CTX_NEED_KMS:
+   case MONGOCRYPT_CTX_READY:
    default:
       return _mongocrypt_ctx_fail_w_msg (ctx, "wrong state");
    }
@@ -536,6 +577,12 @@ mongocrypt_ctx_next_kms_ctx (mongocrypt_ctx_t *ctx)
       return ctx->vtable.next_kms_ctx (ctx);
    case MONGOCRYPT_CTX_ERROR:
       return NULL;
+   case MONGOCRYPT_CTX_DONE:
+   case MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS:
+   case MONGOCRYPT_CTX_NEED_MONGO_COLLINFO:
+   case MONGOCRYPT_CTX_NEED_MONGO_KEYS:
+   case MONGOCRYPT_CTX_NEED_MONGO_MARKINGS:
+   case MONGOCRYPT_CTX_READY:
    default:
       _mongocrypt_ctx_fail_w_msg (ctx, "wrong state");
       return NULL;
@@ -545,8 +592,7 @@ mongocrypt_ctx_next_kms_ctx (mongocrypt_ctx_t *ctx)
 
 bool
 mongocrypt_ctx_provide_kms_providers (
-   mongocrypt_ctx_t *ctx,
-   mongocrypt_binary_t *kms_providers_definition)
+   mongocrypt_ctx_t *ctx, mongocrypt_binary_t *kms_providers_definition)
 {
    if (!ctx) {
       return false;
@@ -562,36 +608,36 @@ mongocrypt_ctx_provide_kms_providers (
       return false;
    }
 
-   if (!_mongocrypt_parse_kms_providers(
-          kms_providers_definition,
-          &ctx->per_ctx_kms_providers,
-          ctx->status,
-          &ctx->crypt->log)) {
+   if (!kms_providers_definition) {
+      _mongocrypt_ctx_fail_w_msg (
+         ctx, "KMS provider credential mapping not provided");
       return false;
    }
 
-   if (!_mongocrypt_opts_kms_providers_validate(
-          &ctx->per_ctx_kms_providers,
-          ctx->status)) {
+   if (!_mongocrypt_parse_kms_providers (kms_providers_definition,
+                                         &ctx->per_ctx_kms_providers,
+                                         ctx->status,
+                                         &ctx->crypt->log)) {
+      return _mongocrypt_ctx_fail (ctx);
+   }
+
+   if (!_mongocrypt_opts_kms_providers_validate (
+          &ctx->crypt->opts, &ctx->per_ctx_kms_providers, ctx->status)) {
       /* Remove the parsed KMS providers if they are invalid */
-      _mongocrypt_opts_kms_providers_cleanup(&ctx->per_ctx_kms_providers);
-      memset(&ctx->per_ctx_kms_providers,
-             0,
-             sizeof(ctx->per_ctx_kms_providers));
-      return false;
+      _mongocrypt_opts_kms_providers_cleanup (&ctx->per_ctx_kms_providers);
+      memset (
+         &ctx->per_ctx_kms_providers, 0, sizeof (ctx->per_ctx_kms_providers));
+      return _mongocrypt_ctx_fail (ctx);
    }
 
-   memcpy(
-      &ctx->kms_providers,
-      &ctx->crypt->opts.kms_providers,
-      sizeof(_mongocrypt_opts_kms_providers_t));
-   _mongocrypt_opts_merge_kms_providers(
-      &ctx->kms_providers,
-      &ctx->per_ctx_kms_providers);
+   memcpy (&ctx->kms_providers,
+           &ctx->crypt->opts.kms_providers,
+           sizeof (_mongocrypt_opts_kms_providers_t));
+   _mongocrypt_opts_merge_kms_providers (&ctx->kms_providers,
+                                         &ctx->per_ctx_kms_providers);
 
-   ctx->state = ctx->kb.state == KB_ADDING_DOCS ?
-      MONGOCRYPT_CTX_NEED_MONGO_KEYS :
-      MONGOCRYPT_CTX_NEED_KMS;
+   ctx->state = ctx->kb.state == KB_ADDING_DOCS ? MONGOCRYPT_CTX_NEED_MONGO_KEYS
+                                                : MONGOCRYPT_CTX_NEED_KMS;
    if (ctx->vtable.after_kms_credentials_provided) {
       return ctx->vtable.after_kms_credentials_provided (ctx);
    }
@@ -618,6 +664,12 @@ mongocrypt_ctx_kms_done (mongocrypt_ctx_t *ctx)
       return ctx->vtable.kms_done (ctx);
    case MONGOCRYPT_CTX_ERROR:
       return false;
+   case MONGOCRYPT_CTX_DONE:
+   case MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS:
+   case MONGOCRYPT_CTX_NEED_MONGO_COLLINFO:
+   case MONGOCRYPT_CTX_NEED_MONGO_KEYS:
+   case MONGOCRYPT_CTX_NEED_MONGO_MARKINGS:
+   case MONGOCRYPT_CTX_READY:
    default:
       return _mongocrypt_ctx_fail_w_msg (ctx, "wrong state");
    }
@@ -635,7 +687,7 @@ mongocrypt_ctx_finalize (mongocrypt_ctx_t *ctx, mongocrypt_binary_t *out)
    }
 
    if (!out) {
-      return _mongocrypt_ctx_fail_w_msg (ctx, "invalid NULL input");
+      return _mongocrypt_ctx_fail_w_msg (ctx, "invalid NULL output");
    }
 
    if (!ctx->vtable.finalize) {
@@ -647,6 +699,12 @@ mongocrypt_ctx_finalize (mongocrypt_ctx_t *ctx, mongocrypt_binary_t *out)
       return ctx->vtable.finalize (ctx, out);
    case MONGOCRYPT_CTX_ERROR:
       return false;
+   case MONGOCRYPT_CTX_DONE:
+   case MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS:
+   case MONGOCRYPT_CTX_NEED_KMS:
+   case MONGOCRYPT_CTX_NEED_MONGO_COLLINFO:
+   case MONGOCRYPT_CTX_NEED_MONGO_KEYS:
+   case MONGOCRYPT_CTX_NEED_MONGO_MARKINGS:
    default:
       return _mongocrypt_ctx_fail_w_msg (ctx, "wrong state");
    }
@@ -659,6 +717,10 @@ mongocrypt_ctx_status (mongocrypt_ctx_t *ctx, mongocrypt_status_t *out)
       return false;
    }
 
+   if (!out) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "invalid NULL output");
+   }
+
    if (!mongocrypt_status_ok (ctx->status)) {
       _mongocrypt_status_copy_to (ctx->status, out);
       return false;
@@ -679,13 +741,15 @@ mongocrypt_ctx_destroy (mongocrypt_ctx_t *ctx)
       ctx->vtable.cleanup (ctx);
    }
 
-   _mongocrypt_opts_kms_providers_cleanup(&ctx->per_ctx_kms_providers);
+   mc_RangeOpts_cleanup (&ctx->opts.rangeopts.value);
+   _mongocrypt_opts_kms_providers_cleanup (&ctx->per_ctx_kms_providers);
    _mongocrypt_kek_cleanup (&ctx->opts.kek);
    mongocrypt_status_destroy (ctx->status);
    _mongocrypt_key_broker_cleanup (&ctx->kb);
    _mongocrypt_buffer_cleanup (&ctx->opts.key_material);
    _mongocrypt_key_alt_name_destroy_all (ctx->opts.key_alt_names);
    _mongocrypt_buffer_cleanup (&ctx->opts.key_id);
+   _mongocrypt_buffer_cleanup (&ctx->opts.index_key_id);
    bson_free (ctx);
    return;
 }
@@ -743,8 +807,10 @@ mongocrypt_ctx_setopt_masterkey_aws (mongocrypt_ctx_t *ctx,
    bson_append_utf8 (&as_bson,
                      MONGOCRYPT_STR_AND_LEN ("provider"),
                      MONGOCRYPT_STR_AND_LEN ("aws"));
+   BSON_ASSERT (region_len <= INT_MAX);
    bson_append_utf8 (
       &as_bson, MONGOCRYPT_STR_AND_LEN ("region"), region, region_len);
+   BSON_ASSERT (cmk_len <= INT_MAX);
    bson_append_utf8 (&as_bson, MONGOCRYPT_STR_AND_LEN ("key"), cmk, cmk_len);
    bin = mongocrypt_binary_new_from_data ((uint8_t *) bson_get_data (&as_bson),
                                           as_bson.len);
@@ -801,6 +867,15 @@ _mongocrypt_ctx_init (mongocrypt_ctx_t *ctx,
 {
    bool has_id = false, has_alt_name = false, has_multiple_alt_names = false;
 
+   BSON_ASSERT_PARAM (ctx);
+   BSON_ASSERT_PARAM (opts_spec);
+
+   // This condition is guarded in setopt_algorithm:
+   BSON_ASSERT (
+      !(ctx->opts.index_type.set &&
+        ctx->opts.algorithm != MONGOCRYPT_ENCRYPTION_ALGORITHM_NONE) &&
+      "Both an encryption algorithm and an index_type were set.");
+
    if (ctx->initialized) {
       return _mongocrypt_ctx_fail_w_msg (ctx, "cannot double initialize");
    }
@@ -825,8 +900,8 @@ _mongocrypt_ctx_init (mongocrypt_ctx_t *ctx,
          return _mongocrypt_ctx_fail_w_msg (ctx, "master key required");
       }
       if (!ctx->crypt->opts.use_need_kms_credentials_state &&
-          !(ctx->opts.kek.kms_provider &
-            _mongocrypt_ctx_kms_providers(ctx)->configured_providers)) {
+          !((int) ctx->opts.kek.kms_provider &
+            _mongocrypt_ctx_kms_providers (ctx)->configured_providers)) {
          return _mongocrypt_ctx_fail_w_msg (
             ctx, "requested kms provider not configured");
       }
@@ -836,6 +911,16 @@ _mongocrypt_ctx_init (mongocrypt_ctx_t *ctx,
       return _mongocrypt_ctx_fail_w_msg (ctx, "master key prohibited");
    }
 
+   /* Check that the kms provider required by the datakey is configured.  */
+   if (ctx->opts.kek.kms_provider) {
+      if (!((ctx->crypt->opts.kms_providers.need_credentials |
+             ctx->crypt->opts.kms_providers.configured_providers) &
+            (int) ctx->opts.kek.kms_provider)) {
+         return _mongocrypt_ctx_fail_w_msg (
+            ctx, "kms provider required by datakey is not configured");
+      }
+   }
+
    /* Special case. key_descriptor applies to explicit encryption. It must be
     * either a key id or *one* key alt name, but not both.
     * key_alt_names applies to creating a data key. It may be one or multiple
@@ -887,6 +972,11 @@ _mongocrypt_ctx_init (mongocrypt_ctx_t *ctx,
       return _mongocrypt_ctx_fail_w_msg (ctx, "algorithm prohibited");
    }
 
+   if (opts_spec->rangeopts == OPT_PROHIBITED && ctx->opts.rangeopts.set) {
+      return _mongocrypt_ctx_fail_w_msg (
+         ctx, "range opts are prohibited on this context");
+   }
+
    _mongocrypt_key_broker_init (&ctx->kb, ctx->crypt);
    return true;
 }
@@ -899,6 +989,8 @@ _mongocrypt_ctx_state_from_key_broker (mongocrypt_ctx_t *ctx)
    mongocrypt_ctx_state_t new_state = MONGOCRYPT_CTX_ERROR;
    bool ret = false;
 
+   BSON_ASSERT_PARAM (ctx);
+
    status = ctx->status;
    kb = &ctx->kb;
 
@@ -924,6 +1016,11 @@ _mongocrypt_ctx_state_from_key_broker (mongocrypt_ctx_t *ctx)
       }
       ret = true;
       break;
+   case KB_ADDING_DOCS_ANY:
+      /* Assume KMS credentials have been provided. */
+      new_state = MONGOCRYPT_CTX_NEED_MONGO_KEYS;
+      ret = true;
+      break;
    case KB_AUTHENTICATING:
    case KB_DECRYPTING_KEY_MATERIAL:
       new_state = MONGOCRYPT_CTX_NEED_KMS;
@@ -938,8 +1035,9 @@ _mongocrypt_ctx_state_from_key_broker (mongocrypt_ctx_t *ctx)
       ret = true;
       break;
    /* As currently implemented, we do not expect to ever be in KB_REQUESTING
-    * state when calling this function. */
+    * or KB_REQUESTING_ANY state when calling this function. */
    case KB_REQUESTING:
+   default:
       CLIENT_ERR ("key broker in unexpected state");
       new_state = MONGOCRYPT_CTX_ERROR;
       ret = false;
@@ -1013,6 +1111,11 @@ mongocrypt_ctx_setopt_key_encryption_key (mongocrypt_ctx_t *ctx,
       return _mongocrypt_ctx_fail_w_msg (ctx, "key encryption key already set");
    }
 
+   if (!bin) {
+      return _mongocrypt_ctx_fail_w_msg (
+         ctx, "invalid NULL key encryption key document");
+   }
+
    if (!_mongocrypt_binary_to_bson (bin, &as_bson)) {
       return _mongocrypt_ctx_fail_w_msg (ctx, "invalid BSON");
    }
@@ -1036,8 +1139,145 @@ mongocrypt_ctx_setopt_key_encryption_key (mongocrypt_ctx_t *ctx,
 }
 
 _mongocrypt_opts_kms_providers_t *
-_mongocrypt_ctx_kms_providers(mongocrypt_ctx_t *ctx) {
-   return ctx->kms_providers.configured_providers ?
-      &ctx->kms_providers :
-      &ctx->crypt->opts.kms_providers;
+_mongocrypt_ctx_kms_providers (mongocrypt_ctx_t *ctx)
+{
+   BSON_ASSERT_PARAM (ctx);
+
+   return ctx->kms_providers.configured_providers
+             ? &ctx->kms_providers
+             : &ctx->crypt->opts.kms_providers;
+}
+
+bool
+mongocrypt_ctx_setopt_contention_factor (mongocrypt_ctx_t *ctx,
+                                         int64_t contention_factor)
+{
+   if (!ctx) {
+      return false;
+   }
+   ctx->opts.contention_factor.value = contention_factor;
+   ctx->opts.contention_factor.set = true;
+   return true;
+}
+
+bool
+mongocrypt_ctx_setopt_index_key_id (mongocrypt_ctx_t *ctx,
+                                    mongocrypt_binary_t *key_id)
+{
+   if (!ctx) {
+      return false;
+   }
+
+   return _set_binary_opt (
+      ctx, key_id, &ctx->opts.index_key_id, BSON_SUBTYPE_UUID);
+}
+
+bool
+mongocrypt_ctx_setopt_query_type (mongocrypt_ctx_t *ctx,
+                                  const char *query_type,
+                                  int len)
+{
+   if (!ctx) {
+      return false;
+   }
+
+   if (ctx->initialized) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "Cannot set options after init");
+   }
+   if (ctx->state == MONGOCRYPT_CTX_ERROR) {
+      return false;
+   }
+   if (len < -1) {
+      return _mongocrypt_ctx_fail_w_msg (ctx,
+                                         "Invalid query_type string length");
+   }
+   if (!query_type) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "Invalid null query_type string");
+   }
+
+   const size_t calc_len = len == -1 ? strlen (query_type) : (size_t) len;
+   mstr_view qt_str = mstrv_view_data (query_type, calc_len);
+   if (mstr_eq_ignore_case (qt_str,
+                            mstrv_lit (MONGOCRYPT_QUERY_TYPE_EQUALITY_STR))) {
+      ctx->opts.query_type.value = MONGOCRYPT_QUERY_TYPE_EQUALITY;
+      ctx->opts.query_type.set = true;
+   } else if (mstr_eq_ignore_case (
+                 qt_str, mstrv_lit (MONGOCRYPT_QUERY_TYPE_RANGEPREVIEW_STR))) {
+      ctx->opts.query_type.value = MONGOCRYPT_QUERY_TYPE_RANGEPREVIEW;
+      ctx->opts.query_type.set = true;
+   } else {
+      /* don't check if qt_str.len fits in int; we want the diagnostic output */
+      char *error = bson_strdup_printf (
+         "Unsupported query_type \"%.*s\"",
+         qt_str.len <= (size_t) INT_MAX ? (int) qt_str.len : INT_MAX,
+         qt_str.data);
+      _mongocrypt_ctx_fail_w_msg (ctx, error);
+      bson_free (error);
+      return false;
+   }
+   return true;
+}
+
+const char *
+_mongocrypt_index_type_to_string (mongocrypt_index_type_t val)
+{
+   switch (val) {
+   case MONGOCRYPT_INDEX_TYPE_NONE:
+      return "None";
+   case MONGOCRYPT_INDEX_TYPE_EQUALITY:
+      return "Equality";
+   case MONGOCRYPT_INDEX_TYPE_RANGEPREVIEW:
+      return "RangePreview";
+   default:
+      return "Unknown";
+   }
+}
+
+const char *
+_mongocrypt_query_type_to_string (mongocrypt_query_type_t val)
+{
+   switch (val) {
+   case MONGOCRYPT_QUERY_TYPE_EQUALITY:
+      return "Equality";
+   case MONGOCRYPT_QUERY_TYPE_RANGEPREVIEW:
+      return "RangePreview";
+   default:
+      return "Unknown";
+   }
+}
+
+bool
+mongocrypt_ctx_setopt_algorithm_range (mongocrypt_ctx_t *ctx,
+                                       mongocrypt_binary_t *opts)
+{
+   bson_t as_bson;
+
+   if (!ctx) {
+      return false;
+   }
+
+   if (ctx->initialized) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "cannot set options after init");
+   }
+
+   if (ctx->state == MONGOCRYPT_CTX_ERROR) {
+      return false;
+   }
+
+   if (ctx->opts.rangeopts.set) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "RangeOpts already set");
+   }
+
+
+   if (!_mongocrypt_binary_to_bson (opts, &as_bson)) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "invalid BSON");
+   }
+
+   if (!mc_RangeOpts_parse (
+          &ctx->opts.rangeopts.value, &as_bson, ctx->status)) {
+      return _mongocrypt_ctx_fail (ctx);
+   }
+
+   ctx->opts.rangeopts.set = true;
+   return true;
 }
