@@ -20,6 +20,9 @@
 void
 _mongocrypt_key_broker_init (_mongocrypt_key_broker_t *kb, mongocrypt_t *crypt)
 {
+   BSON_ASSERT_PARAM (kb);
+   BSON_ASSERT_PARAM (crypt);
+
    memset (kb, 0, sizeof (*kb));
    kb->crypt = crypt;
    kb->state = KB_REQUESTING;
@@ -39,7 +42,9 @@ _key_returned_prepend (_mongocrypt_key_broker_t *kb,
 {
    key_returned_t *key_returned;
 
-   BSON_ASSERT (key_doc);
+   BSON_ASSERT_PARAM (kb);
+   BSON_ASSERT_PARAM (list);
+   BSON_ASSERT_PARAM (key_doc);
 
    key_returned = bson_malloc0 (sizeof (*key_returned));
    BSON_ASSERT (key_returned);
@@ -65,14 +70,20 @@ _key_returned_find_one (key_returned_t *list,
 {
    key_returned_t *key_returned;
 
+   /* list can be NULL. */
+   /* key_id and key_alt_names are not dereferenced in this function and they
+    * are checked just before being passed on as parameters. */
+
    for (key_returned = list; NULL != key_returned;
         key_returned = key_returned->next) {
       if (key_id) {
+         BSON_ASSERT (key_returned->doc);
          if (0 == _mongocrypt_buffer_cmp (key_id, &key_returned->doc->id)) {
             return key_returned;
          }
       }
       if (key_alt_names) {
+         BSON_ASSERT (key_returned->doc);
          if (_mongocrypt_key_alt_name_intersects (
                 key_alt_names, key_returned->doc->key_alt_names)) {
             return key_returned;
@@ -92,6 +103,10 @@ _key_request_find_one (_mongocrypt_key_broker_t *kb,
 {
    key_request_t *key_request;
 
+   BSON_ASSERT_PARAM (kb);
+   /* key_id and key_alt_names are not dereferenced in this function and they
+    * are checked just before being passed on as parameters. */
+
    for (key_request = kb->key_requests; NULL != key_request;
         key_request = key_request->next) {
       if (key_id) {
@@ -115,6 +130,8 @@ _all_key_requests_satisfied (_mongocrypt_key_broker_t *kb)
 {
    key_request_t *key_request;
 
+   BSON_ASSERT_PARAM (kb);
+
    for (key_request = kb->key_requests; NULL != key_request;
         key_request = key_request->next) {
       if (!key_request->satisfied) {
@@ -129,15 +146,20 @@ _key_broker_fail_w_msg (_mongocrypt_key_broker_t *kb, const char *msg)
 {
    mongocrypt_status_t *status;
 
+   BSON_ASSERT_PARAM (kb);
+   BSON_ASSERT_PARAM (msg);
+
    kb->state = KB_ERROR;
    status = kb->status;
-   CLIENT_ERR (msg);
+   CLIENT_ERR ("%s", msg);
    return false;
 }
 
 static bool
 _key_broker_fail (_mongocrypt_key_broker_t *kb)
 {
+   BSON_ASSERT_PARAM (kb);
+
    if (mongocrypt_status_ok (kb->status)) {
       return _key_broker_fail_w_msg (
          kb, "unexpected, failing but no error status set");
@@ -153,7 +175,10 @@ _try_satisfying_from_cache (_mongocrypt_key_broker_t *kb, key_request_t *req)
    _mongocrypt_cache_key_value_t *value = NULL;
    bool ret = false;
 
-   if (kb->state != KB_REQUESTING) {
+   BSON_ASSERT_PARAM (kb);
+   BSON_ASSERT_PARAM (req);
+
+   if (kb->state != KB_REQUESTING && kb->state != KB_ADDING_DOCS_ANY) {
       _key_broker_fail_w_msg (
          kb, "trying to retrieve key from cache in invalid state");
       goto cleanup;
@@ -202,6 +227,9 @@ _store_to_cache (_mongocrypt_key_broker_t *kb, key_returned_t *key_returned)
    _mongocrypt_cache_key_attr_t *attr;
    bool ret;
 
+   BSON_ASSERT_PARAM (kb);
+   BSON_ASSERT_PARAM (key_returned);
+
    if (!key_returned->decrypted) {
       return _key_broker_fail_w_msg (kb, "cannot cache non-decrypted key");
    }
@@ -229,6 +257,9 @@ _mongocrypt_key_broker_request_id (_mongocrypt_key_broker_t *kb,
 {
    key_request_t *req;
 
+   BSON_ASSERT_PARAM (kb);
+   BSON_ASSERT_PARAM (key_id);
+
    if (kb->state != KB_REQUESTING) {
       return _key_broker_fail_w_msg (
          kb, "attempting to request a key id, but in wrong state");
@@ -262,6 +293,9 @@ _mongocrypt_key_broker_request_name (_mongocrypt_key_broker_t *kb,
    key_request_t *req;
    _mongocrypt_key_alt_name_t *key_alt_name;
 
+   BSON_ASSERT_PARAM (kb);
+   BSON_ASSERT_PARAM (key_alt_name_value);
+
    if (kb->state != KB_REQUESTING) {
       return _key_broker_fail_w_msg (
          kb, "attempting to request a key name, but in wrong state");
@@ -287,17 +321,37 @@ _mongocrypt_key_broker_request_name (_mongocrypt_key_broker_t *kb,
    return true;
 }
 
+bool
+_mongocrypt_key_broker_request_any (_mongocrypt_key_broker_t *kb)
+{
+   BSON_ASSERT_PARAM (kb);
+
+   if (kb->state != KB_REQUESTING) {
+      return _key_broker_fail_w_msg (
+         kb, "attempting to request any keys, but in wrong state");
+   }
+
+   if (kb->key_requests) {
+      return _key_broker_fail_w_msg (
+         kb, "attempting to request any keys, but requests already made");
+   }
+
+   kb->state = KB_ADDING_DOCS_ANY;
+
+   return true;
+}
+
 bool
 _mongocrypt_key_broker_requests_done (_mongocrypt_key_broker_t *kb)
 {
+   BSON_ASSERT_PARAM (kb);
+
    if (kb->state != KB_REQUESTING) {
       return _key_broker_fail_w_msg (
          kb, "attempting to finish adding requests, but in wrong state");
    }
 
    if (kb->key_requests) {
-      /* If all were satisfied from the cache, then we're done since those all
-       * have decrypted material */
       if (_all_key_requests_satisfied (kb)) {
          kb->state = KB_DONE;
       } else {
@@ -320,7 +374,8 @@ _mongocrypt_key_broker_filter (_mongocrypt_key_broker_t *kb,
    bson_t ids, names;
    bson_t *filter;
 
-   BSON_ASSERT (kb);
+   BSON_ASSERT_PARAM (kb);
+   BSON_ASSERT_PARAM (out);
 
    if (kb->state != KB_ADDING_DOCS) {
       return _key_broker_fail_w_msg (
@@ -346,8 +401,7 @@ _mongocrypt_key_broker_filter (_mongocrypt_key_broker_t *kb,
 
          key_str = bson_strdup_printf ("%d", id_index++);
          if (!key_str ||
-             !_mongocrypt_buffer_append (
-                &req->id, &ids, key_str, (uint32_t) strlen (key_str))) {
+             !_mongocrypt_buffer_append (&req->id, &ids, key_str, -1)) {
             bson_destroy (&ids);
             bson_destroy (&names);
             bson_free (key_str);
@@ -362,11 +416,12 @@ _mongocrypt_key_broker_filter (_mongocrypt_key_broker_t *kb,
            key_alt_name = key_alt_name->next) {
          char *key_str;
 
+
          key_str = bson_strdup_printf ("%d", name_index++);
          BSON_ASSERT (key_str);
          if (!bson_append_value (&names,
                                  key_str,
-                                 (uint32_t) strlen (key_str),
+                                 (int) strlen (key_str),
                                  &key_alt_name->value)) {
             bson_destroy (&ids);
             bson_destroy (&names);
@@ -423,7 +478,10 @@ _mongocrypt_key_broker_add_doc (_mongocrypt_key_broker_t *kb,
    _mongocrypt_kms_provider_t kek_provider;
    char *access_token = NULL;
 
-   if (kb->state != KB_ADDING_DOCS) {
+   BSON_ASSERT_PARAM (kb);
+   BSON_ASSERT_PARAM (kms_providers);
+
+   if (kb->state != KB_ADDING_DOCS && kb->state != KB_ADDING_DOCS_ANY) {
       _key_broker_fail_w_msg (
          kb, "attempting to add a key doc, but in wrong state");
       goto done;
@@ -445,11 +503,37 @@ _mongocrypt_key_broker_add_doc (_mongocrypt_key_broker_t *kb,
       goto done;
    }
 
-   /* Ensure that this document matches at least one request. */
    if (!_key_request_find_one (kb, &key_doc->id, key_doc->key_alt_names)) {
-      _key_broker_fail_w_msg (
-         kb, "unexpected key returned, does not match any requests");
-      goto done;
+      /* If in normal mode, ensure that this document matches at least one
+       * existing request. */
+      if (kb->state == KB_ADDING_DOCS) {
+         _key_broker_fail_w_msg (
+            kb, "unexpected key returned, does not match any requests");
+         goto done;
+      }
+
+      /* If in any mode, add request for provided document now. */
+      if (kb->state == KB_ADDING_DOCS_ANY) {
+         key_request_t *const req = bson_malloc0 (sizeof (key_request_t));
+
+         BSON_ASSERT (req);
+
+         _mongocrypt_buffer_copy_to (&key_doc->id, &req->id);
+         req->alt_name =
+            _mongocrypt_key_alt_name_copy_all (key_doc->key_alt_names);
+         req->next = kb->key_requests;
+         kb->key_requests = req;
+
+         if (!_try_satisfying_from_cache (kb, req)) {
+            goto done;
+         }
+
+         /* Key is already cached; no work to be done. */
+         if (req->satisfied) {
+            ret = true;
+            goto done;
+         }
+      }
    }
 
    /* Check if there are other keys_returned with intersecting altnames or
@@ -465,7 +549,7 @@ _mongocrypt_key_broker_add_doc (_mongocrypt_key_broker_t *kb,
 
    /* Check that the returned key doc's provider matches. */
    kek_provider = key_doc->kek.kms_provider;
-   if (0 == (kek_provider & kms_providers->configured_providers)) {
+   if (0 == ((int) kek_provider & kms_providers->configured_providers)) {
       _key_broker_fail_w_msg (
          kb, "client not configured with KMS provider necessary to decrypt");
       goto done;
@@ -473,6 +557,7 @@ _mongocrypt_key_broker_add_doc (_mongocrypt_key_broker_t *kb,
 
    /* If the KMS provider is local, decrypt immediately. Otherwise, create the
     * HTTP KMS request. */
+   BSON_ASSERT (kb->crypt);
    if (kek_provider == MONGOCRYPT_KMS_PROVIDER_LOCAL) {
       if (!_mongocrypt_unwrap_key (kb->crypt->crypto,
                                    &kms_providers->local.key,
@@ -497,7 +582,12 @@ _mongocrypt_key_broker_add_doc (_mongocrypt_key_broker_t *kb,
          goto done;
       }
    } else if (kek_provider == MONGOCRYPT_KMS_PROVIDER_AZURE) {
-      access_token = _mongocrypt_cache_oauth_get (kb->crypt->cache_oauth_azure);
+      if (kms_providers->azure.access_token) {
+         access_token = bson_strdup (kms_providers->azure.access_token);
+      } else {
+         access_token =
+            _mongocrypt_cache_oauth_get (kb->crypt->cache_oauth_azure);
+      }
       if (!access_token) {
          key_returned->needs_auth = true;
          /* Create an oauth request if one does not exist. */
@@ -527,7 +617,12 @@ _mongocrypt_key_broker_add_doc (_mongocrypt_key_broker_t *kb,
          }
       }
    } else if (kek_provider == MONGOCRYPT_KMS_PROVIDER_GCP) {
-      access_token = _mongocrypt_cache_oauth_get (kb->crypt->cache_oauth_gcp);
+      if (NULL != kms_providers->gcp.access_token) {
+         access_token = bson_strdup (kms_providers->gcp.access_token);
+      } else {
+         access_token =
+            _mongocrypt_cache_oauth_get (kb->crypt->cache_oauth_gcp);
+      }
       if (!access_token) {
          key_returned->needs_auth = true;
          /* Create an oauth request if one does not exist. */
@@ -615,7 +710,9 @@ _mongocrypt_key_broker_docs_done (_mongocrypt_key_broker_t *kb)
    bool needs_decryption;
    bool needs_auth;
 
-   if (kb->state != KB_ADDING_DOCS) {
+   BSON_ASSERT_PARAM (kb);
+
+   if (kb->state != KB_ADDING_DOCS && kb->state != KB_ADDING_DOCS_ANY) {
       return _key_broker_fail_w_msg (
          kb, "attempting to finish adding docs, but in wrong state");
    }
@@ -660,6 +757,8 @@ _mongocrypt_key_broker_docs_done (_mongocrypt_key_broker_t *kb)
 mongocrypt_kms_ctx_t *
 _mongocrypt_key_broker_next_kms (_mongocrypt_key_broker_t *kb)
 {
+   BSON_ASSERT_PARAM (kb);
+
    if (kb->state != KB_DECRYPTING_KEY_MATERIAL &&
        kb->state != KB_AUTHENTICATING) {
       _key_broker_fail_w_msg (
@@ -710,10 +809,13 @@ _mongocrypt_key_broker_next_kms (_mongocrypt_key_broker_t *kb)
 bool
 _mongocrypt_key_broker_kms_done (
    _mongocrypt_key_broker_t *kb,
-   _mongocrypt_opts_kms_providers_t* kms_providers)
+   _mongocrypt_opts_kms_providers_t *kms_providers)
 {
    key_returned_t *key_returned;
 
+   BSON_ASSERT_PARAM (kb);
+   BSON_ASSERT_PARAM (kms_providers);
+
    if (kb->state != KB_DECRYPTING_KEY_MATERIAL &&
        kb->state != KB_AUTHENTICATING) {
       return _key_broker_fail_w_msg (
@@ -767,8 +869,12 @@ _mongocrypt_key_broker_kms_done (
 
          if (key_returned->doc->kek.kms_provider ==
              MONGOCRYPT_KMS_PROVIDER_AZURE) {
-            access_token =
-               _mongocrypt_cache_oauth_get (kb->crypt->cache_oauth_azure);
+            if (kms_providers->azure.access_token) {
+               access_token = bson_strdup (kms_providers->azure.access_token);
+            } else {
+               access_token =
+                  _mongocrypt_cache_oauth_get (kb->crypt->cache_oauth_azure);
+            }
 
             if (!access_token) {
                return _key_broker_fail_w_msg (
@@ -789,8 +895,12 @@ _mongocrypt_key_broker_kms_done (
             bson_free (access_token);
          } else if (key_returned->doc->kek.kms_provider ==
                     MONGOCRYPT_KMS_PROVIDER_GCP) {
-            access_token =
-               _mongocrypt_cache_oauth_get (kb->crypt->cache_oauth_gcp);
+            if (NULL != kms_providers->gcp.access_token) {
+               access_token = bson_strdup (kms_providers->gcp.access_token);
+            } else {
+               access_token =
+                  _mongocrypt_cache_oauth_get (kb->crypt->cache_oauth_gcp);
+            }
 
             if (!access_token) {
                return _key_broker_fail_w_msg (
@@ -829,10 +939,9 @@ _mongocrypt_key_broker_kms_done (
              MONGOCRYPT_KMS_PROVIDER_AZURE ||
           key_returned->doc->kek.kms_provider == MONGOCRYPT_KMS_PROVIDER_GCP) {
          if (key_returned->decrypted) {
-            return _key_broker_fail_w_msg (
-               kb,
-               "unexpected, returned keys should not be "
-               "decrypted before KMS completion");
+            /* Non-local keys may have been decrypted previously if the key
+             * broker has been restarted. */
+            continue;
          }
 
          if (!key_returned->kms.req) {
@@ -885,7 +994,7 @@ _mongocrypt_key_broker_kms_done (
 }
 
 
-bool
+static bool
 _get_decrypted_key_material (_mongocrypt_key_broker_t *kb,
                              _mongocrypt_buffer_t *key_id,
                              _mongocrypt_key_alt_name_t *key_alt_name,
@@ -894,6 +1003,12 @@ _get_decrypted_key_material (_mongocrypt_key_broker_t *kb,
 {
    key_returned_t *key_returned;
 
+   BSON_ASSERT_PARAM (kb);
+   /* key_id can be NULL */
+   /* key_alt_name can be NULL */
+   BSON_ASSERT_PARAM (out);
+   /* key_id_out is checked before each use, so it can be NULL */
+
    _mongocrypt_buffer_init (out);
    if (key_id_out) {
       _mongocrypt_buffer_init (key_id_out);
@@ -928,7 +1043,11 @@ _mongocrypt_key_broker_decrypted_key_by_id (_mongocrypt_key_broker_t *kb,
                                             const _mongocrypt_buffer_t *key_id,
                                             _mongocrypt_buffer_t *out)
 {
-   if (kb->state != KB_DONE) {
+   BSON_ASSERT_PARAM (kb);
+   BSON_ASSERT_PARAM (key_id);
+   BSON_ASSERT_PARAM (out);
+
+   if (kb->state != KB_DONE && kb->state != KB_REQUESTING) {
       return _key_broker_fail_w_msg (
          kb, "attempting retrieve decrypted key material, but in wrong state");
    }
@@ -949,6 +1068,11 @@ _mongocrypt_key_broker_decrypted_key_by_name (
    bool ret;
    _mongocrypt_key_alt_name_t *key_alt_name;
 
+   BSON_ASSERT_PARAM (kb);
+   BSON_ASSERT_PARAM (key_alt_name_value);
+   BSON_ASSERT_PARAM (out);
+   BSON_ASSERT_PARAM (key_id_out);
+
    if (kb->state != KB_DONE) {
       return _key_broker_fail_w_msg (
          kb, "attempting retrieve decrypted key material, but in wrong state");
@@ -964,7 +1088,8 @@ bool
 _mongocrypt_key_broker_status (_mongocrypt_key_broker_t *kb,
                                mongocrypt_status_t *out)
 {
-   BSON_ASSERT (kb);
+   BSON_ASSERT_PARAM (kb);
+   BSON_ASSERT_PARAM (out);
 
    if (!mongocrypt_status_ok (kb->status)) {
       _mongocrypt_status_copy_to (kb->status, out);
@@ -1011,6 +1136,9 @@ _destroy_keys_returned (key_returned_t *head)
 void
 _mongocrypt_key_broker_cleanup (_mongocrypt_key_broker_t *kb)
 {
+   if (!kb) {
+      return;
+   }
    mongocrypt_status_destroy (kb->status);
    _mongocrypt_buffer_cleanup (&kb->filter);
    /* Delete all linked lists */
@@ -1028,7 +1156,9 @@ _mongocrypt_key_broker_add_test_key (_mongocrypt_key_broker_t *kb,
    key_returned_t *key_returned;
    _mongocrypt_key_doc_t *key_doc;
 
-   BSON_ASSERT (kb);
+   BSON_ASSERT_PARAM (kb);
+   BSON_ASSERT_PARAM (key_id);
+
    key_doc = _mongocrypt_key_new ();
    _mongocrypt_buffer_copy_to (key_id, &key_doc->id);
 
@@ -1042,3 +1172,18 @@ _mongocrypt_key_broker_add_test_key (_mongocrypt_key_broker_t *kb,
    /* Hijack state and move directly to DONE. */
    kb->state = KB_DONE;
 }
+
+
+bool
+_mongocrypt_key_broker_restart (_mongocrypt_key_broker_t *kb)
+{
+   BSON_ASSERT_PARAM (kb);
+   if (kb->state != KB_DONE) {
+      return _key_broker_fail_w_msg (
+         kb, "_mongocrypt_key_broker_restart called in wrong state");
+   }
+   kb->state = KB_REQUESTING;
+   _mongocrypt_buffer_cleanup (&kb->filter);
+   _mongocrypt_buffer_init (&kb->filter);
+   return true;
+}
