@@ -31,7 +31,9 @@
 #include "mongocrypt-crypto-private.h"
 #include "mongocrypt-cache-oauth-private.h"
 
-#include "mongo_csfle-v1.h"
+#include "mongo_crypt-v1.h"
+
+#include <mlib/macros.h>
 
 
 #define MONGOCRYPT_GENERIC_ERROR_CODE 1
@@ -64,7 +66,16 @@ tmp_json (const bson_t *bson);
 const char *
 tmp_buf (const _mongocrypt_buffer_t *buf);
 
-
+// _mongocrypt_set_error sets an error status.
+// `status->message` is set after evaluating `format` and variadic args.
+// It is safe to uses a `status->message` as an argument to wrap an error. For example:
+// _mongocrypt_set_error (
+//    status,
+//    MONGOCRYPT_STATUS_ERROR_CLIENT,
+//    MONGOCRYPT_GENERIC_ERROR_CODE,
+//    "Error occurred. Original error: %s",
+//    mongocrypt_status_message (status, NULL /* len */));
+MLIB_ANNOTATE_PRINTF (4, 5)
 void
 _mongocrypt_set_error (mongocrypt_status_t *status,
                        mongocrypt_status_type_t type,
@@ -72,37 +83,37 @@ _mongocrypt_set_error (mongocrypt_status_t *status,
                        const char *format,
                        ...);
 
-typedef struct _mcr_csfle_v1_vtable {
+typedef struct _mongo_crypt_v1_vtable {
 #define MONGOC_CSFLE_FUNCTIONS_X                                             \
    /* status methods */                                                      \
-   X_FUNC (status_create, mongo_csfle_v1_status *, void)                     \
-   X_FUNC (status_destroy, void, mongo_csfle_v1_status *status)              \
-   X_FUNC (status_get_error, int, const mongo_csfle_v1_status *status)       \
+   X_FUNC (status_create, mongo_crypt_v1_status *, void)                     \
+   X_FUNC (status_destroy, void, mongo_crypt_v1_status *status)              \
+   X_FUNC (status_get_error, int, const mongo_crypt_v1_status *status)       \
    X_FUNC (status_get_explanation,                                           \
            const char *,                                                     \
-           const mongo_csfle_v1_status *status)                              \
-   X_FUNC (status_get_code, int, const mongo_csfle_v1_status *status)        \
+           const mongo_crypt_v1_status *status)                              \
+   X_FUNC (status_get_code, int, const mongo_crypt_v1_status *status)        \
    /* lib methods */                                                         \
-   X_FUNC (lib_create, mongo_csfle_v1_lib *, mongo_csfle_v1_status *status)  \
+   X_FUNC (lib_create, mongo_crypt_v1_lib *, mongo_crypt_v1_status *status)  \
    X_FUNC (lib_destroy,                                                      \
            int,                                                              \
-           mongo_csfle_v1_lib *lib,                                          \
-           mongo_csfle_v1_status *status)                                    \
+           mongo_crypt_v1_lib *lib,                                          \
+           mongo_crypt_v1_status *status)                                    \
    /* query_analyzer methods */                                              \
    X_FUNC (query_analyzer_create,                                            \
-           mongo_csfle_v1_query_analyzer *,                                  \
-           mongo_csfle_v1_lib *lib,                                          \
-           mongo_csfle_v1_status *status)                                    \
+           mongo_crypt_v1_query_analyzer *,                                  \
+           mongo_crypt_v1_lib *lib,                                          \
+           mongo_crypt_v1_status *status)                                    \
    X_FUNC (                                                                  \
-      query_analyzer_destroy, void, mongo_csfle_v1_query_analyzer *analyzer) \
+      query_analyzer_destroy, void, mongo_crypt_v1_query_analyzer *analyzer) \
    X_FUNC (analyze_query,                                                    \
            uint8_t *,                                                        \
-           mongo_csfle_v1_query_analyzer *analyer,                           \
+           mongo_crypt_v1_query_analyzer *analyer,                           \
            const uint8_t *documentBSON,                                      \
            const char *ns_str,                                               \
            uint32_t ns_len,                                                  \
            uint32_t *bson_len,                                               \
-           mongo_csfle_v1_status *status)                                    \
+           mongo_crypt_v1_status *status)                                    \
    X_FUNC (get_version, uint64_t, void)                                      \
    X_FUNC (get_version_str, const char *, void)                              \
    /* Free bson data created by csfle */                                     \
@@ -112,8 +123,9 @@ typedef struct _mcr_csfle_v1_vtable {
    MONGOC_CSFLE_FUNCTIONS_X
 #undef X_FUNC
 
-} _mcr_csfle_v1_vtable;
-
+   /// Whether the vtable is valid and complete
+   bool okay;
+} _mongo_crypt_v1_vtable;
 
 struct _mongocrypt_t {
    bool initialized;
@@ -129,9 +141,10 @@ struct _mongocrypt_t {
    uint32_t ctx_counter;
    _mongocrypt_cache_oauth_t *cache_oauth_azure;
    _mongocrypt_cache_oauth_t *cache_oauth_gcp;
-   /// A CSFLE DLL, initialized by mongocrypt_init
-   mcr_dll csfle_lib;
-   _mcr_csfle_v1_vtable csfle_vtable;
+   /// A CSFLE DLL vtable, initialized by mongocrypt_init
+   _mongo_crypt_v1_vtable csfle;
+   /// Pointer to the global csfle_lib object. Should not be freed directly.
+   mongo_crypt_v1_lib *csfle_lib;
 };
 
 typedef enum {
@@ -140,6 +153,16 @@ typedef enum {
    MONGOCRYPT_ENCRYPTION_ALGORITHM_RANDOM = 2
 } mongocrypt_encryption_algorithm_t;
 
+typedef enum {
+   MONGOCRYPT_FLE2_PLACEHOLDER_TYPE_INSERT = 1,
+   MONGOCRYPT_FLE2_PLACEHOLDER_TYPE_FIND = 2
+} mongocrypt_fle2_placeholder_type_t;
+
+typedef enum {
+   MONGOCRYPT_FLE2_ALGORITHM_UNINDEXED = 1,
+   MONGOCRYPT_FLE2_ALGORITHM_EQUALITY = 2,
+   MONGOCRYPT_FLE2_ALGORITHM_RANGE = 3
+} mongocrypt_fle2_encryption_algorithm_t;
 
 bool
 _mongocrypt_validate_and_copy_string (const char *in,
