@@ -22,6 +22,35 @@
 #include <bson/bson.h>
 #include <mongocrypt.h>
 
+static void
+_log_to_stderr (mongocrypt_log_level_t level,
+                const char *message,
+                uint32_t message_len,
+                void *ctx)
+{
+   switch (level) {
+   case MONGOCRYPT_LOG_LEVEL_FATAL:
+      fprintf (stderr, "FATAL");
+      break;
+   case MONGOCRYPT_LOG_LEVEL_ERROR:
+      fprintf (stderr, "ERROR");
+      break;
+   case MONGOCRYPT_LOG_LEVEL_WARNING:
+      fprintf (stderr, "WARNING");
+      break;
+   case MONGOCRYPT_LOG_LEVEL_INFO:
+      fprintf (stderr, "INFO");
+      break;
+   case MONGOCRYPT_LOG_LEVEL_TRACE:
+      fprintf (stderr, "TRACE");
+      break;
+   default:
+      fprintf (stderr, "UNKNOWN");
+      break;
+   }
+   fprintf (stderr, " %s\n", message);
+}
+
 static void
 _load_json_as_bson (const char *path, bson_t *as_bson)
 {
@@ -194,9 +223,6 @@ _run_state_machine (mongocrypt_ctx_t *ctx, bson_t *result)
          bson_free (data);
          CHECK (mongocrypt_ctx_mongo_done (ctx));
          break;
-      case MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS:
-         BSON_ASSERT (0);
-         break;
       case MONGOCRYPT_CTX_NEED_KMS:
          while ((kms = mongocrypt_ctx_next_kms_ctx (ctx))) {
             output = mongocrypt_binary_new ();
@@ -235,6 +261,12 @@ _run_state_machine (mongocrypt_ctx_t *ctx, bson_t *result)
          printf ("\ngot error: %s\n", mongocrypt_status_message (status, NULL));
          abort ();
          break;
+      case MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS:
+         // We don't handle KMS credentials
+         // fallthrough
+      default:
+         BSON_ASSERT (0);
+         break;
       }
    }
 
@@ -271,6 +303,7 @@ main ()
    printf ("******* ENCRYPTION *******\n\n");
    crypt = mongocrypt_new ();
    mongocrypt_setopt_kms_provider_aws (crypt, "example", -1, "example", -1);
+   mongocrypt_setopt_log_handler (crypt, _log_to_stderr, NULL);
    if (!mongocrypt_init (crypt)) {
       fprintf (stderr, "failed to initialize");
       abort ();
@@ -305,8 +338,7 @@ main ()
    key_id = _iter_to_binary (&iter);
    mongocrypt_ctx_setopt_key_id (ctx, key_id);
    bson_destroy (&key_doc);
-   mongocrypt_ctx_setopt_algorithm (
-      ctx, "AEAD_AES_256_CBC_HMAC_SHA_512-Random", -1);
+   mongocrypt_ctx_setopt_algorithm (ctx, MONGOCRYPT_ALGORITHM_RANDOM_STR, -1);
 
    wrapped = BCON_NEW ("v", "hello");
    msg = mongocrypt_binary_new_from_data ((uint8_t *) bson_get_data (wrapped),
