@@ -96,6 +96,8 @@
    do {                                                                    \
       const char *_str_a = (_expr_a);                                      \
       const char *_str_b = (_expr_b);                                      \
+      ASSERT (_str_a);                                                     \
+      ASSERT (_str_b);                                                     \
       int _ret = strcmp (_str_a, _str_b);                                  \
       if (_ret != 0) {                                                     \
          TEST_ERROR ("strings not equal:\n%s\nvs.\n%s\n", _str_a, _str_b); \
@@ -136,16 +138,66 @@
    ASSERT_CMPBYTES (                    \
       (expected).data, (expected).len, (actual).data, (actual).len)
 
-#define ASSERT_CMPINT(_a, _operator, _b)                                \
-   do {                                                                 \
-      int _a_int = _a;                                                  \
-      int _b_int = _b;                                                  \
-      if (!(_a_int _operator _b_int)) {                                 \
-         TEST_ERROR (                                                   \
-            "comparison failed: %d %s %d", _a_int, #_operator, _b_int); \
-      }                                                                 \
+#define ASSERT_CMP_HELPER(_a, _operator, _b, fmt, type)      \
+   do {                                                      \
+      type _a_value = (_a);                                  \
+      type _b_value = (_b);                                  \
+      if (!(_a_value _operator _b_value)) {                  \
+         TEST_ERROR ("comparison failed: %" fmt " %s %" fmt, \
+                     _a_value,                               \
+                     #_operator,                             \
+                     _b_value);                              \
+      }                                                      \
    } while (0);
 
+#define ASSERT_CMPINT(a, eq, b) ASSERT_CMP_HELPER (a, eq, b, "d", int)
+#define ASSERT_CMPUINT(a, eq, b) ASSERT_CMP_HELPER (a, eq, b, "u", unsigned int)
+#define ASSERT_CMPLONG(a, eq, b) ASSERT_CMP_HELPER (a, eq, b, "ld", long)
+#define ASSERT_CMPULONG(a, eq, b) \
+   ASSERT_CMP_HELPER (a, eq, b, "lu", unsigned long)
+#define ASSERT_CMPINT32(a, eq, b) ASSERT_CMP_HELPER (a, eq, b, PRId32, int32_t)
+#define ASSERT_CMPINT64(a, eq, b) ASSERT_CMP_HELPER (a, eq, b, PRId64, int64_t)
+#define ASSERT_CMPUINT16(a, eq, b) \
+   ASSERT_CMP_HELPER (a, eq, b, PRIu16, uint16_t)
+#define ASSERT_CMPUINT32(a, eq, b) \
+   ASSERT_CMP_HELPER (a, eq, b, PRIu32, uint32_t)
+#define ASSERT_CMPUINT64(a, eq, b) \
+   ASSERT_CMP_HELPER (a, eq, b, PRIu64, uint64_t)
+#define ASSERT_CMPSIZE_T(a, eq, b) ASSERT_CMP_HELPER (a, eq, b, "zu", size_t)
+#define ASSERT_CMPSSIZE_T(a, eq, b) ASSERT_CMP_HELPER (a, eq, b, "zd", ssize_t)
+#define ASSERT_CMPDOUBLE(a, eq, b) ASSERT_CMP_HELPER (a, eq, b, "f", double)
+#define ASSERT_CMPPTR(a, eq, b) ASSERT_CMP_HELPER (a, eq, b, "p", const void *)
+
+#define ASSERT_CMPINT128_EQ(A, B)                              \
+   if (1) {                                                    \
+      mlib_int128 left = (A);                                  \
+      mlib_int128 right = (B);                                 \
+      const bool result = mlib_int128_eq (left, right);        \
+      if (!result) {                                           \
+         TEST_ERROR ("Comparison failed: %s == %s (%s != %s)", \
+                     #A,                                       \
+                     #B,                                       \
+                     mlib_int128_format (left).str,            \
+                     mlib_int128_format (right).str);          \
+      }                                                        \
+   } else                                                      \
+      ((void) 0)
+
+#define ASSERT_EQUAL_BSON(expected, actual)                                   \
+   do {                                                                       \
+      bson_t *_expected_bson = expected, *_actual_bson = actual;              \
+      char *_expected_str, *_actual_str;                                      \
+      _expected_str = bson_as_canonical_extended_json (_expected_bson, NULL); \
+      _actual_str = bson_as_canonical_extended_json (_actual_bson, NULL);     \
+      if (!bson_equal (_expected_bson, _actual_bson)) {                       \
+         TEST_ERROR ("BSON unequal.\nExpected: %s\n     Got: %s",             \
+                     _expected_str,                                           \
+                     _actual_str);                                            \
+      }                                                                       \
+      bson_free (_actual_str);                                                \
+      bson_free (_expected_str);                                              \
+   } while (0)
+
 #define ASSERT_MONGOCRYPT_BINARY_EQUAL_BSON(expected, actual)                  \
    do {                                                                        \
       bson_t _expected_bson, _actual_bson;                                     \
