@@ -27,10 +27,26 @@
       "HTTP line 1\r\n",
       "HTTP line 2\r\n"
    ],
-   expect: "ok" | "error message"
+   expect: "ok" | "error message" | <array of strings>
 }
 */
 
+static void
+_satisfy_oauth_request (mongocrypt_kms_ctx_t *kms_ctx)
+{
+   const char *valid_reply =
+      "HTTP/1.1 200 OK\r\n"
+      "Content-Length: 85\r\n"
+      "\r\n"
+      "{\"token_type\":\"Bearer\",\"expires_in\":3599,\"ext_expires_"
+      "in\":3599,\"access_token\":\"AAAA\"}";
+
+   mongocrypt_binary_t *bin = mongocrypt_binary_new_from_data (
+      (uint8_t *) valid_reply, (uint32_t) strlen (valid_reply));
+   ASSERT_OK (mongocrypt_kms_ctx_feed (kms_ctx, bin), kms_ctx);
+   mongocrypt_binary_destroy (bin);
+}
+
 static void
 _test_one_kms_response (_mongocrypt_tester_t *tester, bson_t *test)
 {
@@ -49,7 +65,7 @@ _test_one_kms_response (_mongocrypt_tester_t *tester, bson_t *test)
       bool ok = false;
       mongocrypt_status_t *status;
       mongocrypt_binary_t *bin = NULL;
-      const char *expect;
+      char *expect;
 
       status = mongocrypt_status_new ();
       crypt = _mongocrypt_tester_mongocrypt (TESTER_MONGOCRYPT_DEFAULT);
@@ -66,23 +82,27 @@ _test_one_kms_response (_mongocrypt_tester_t *tester, bson_t *test)
          bin = _mongocrypt_tester_encrypted_doc (tester);
          tester->paths.key_file = "./test/example/key-document.json";
          ASSERT_OK (mongocrypt_ctx_decrypt_init (ctx, bin), ctx);
-      } else if (0 == strcmp ("azure_oauth_datakey", ctx_type)) {
+      } else if (0 == strcmp ("azure_oauth_datakey", ctx_type) ||
+                 0 == strcmp ("azure_datakey", ctx_type)) {
          mongocrypt_ctx_setopt_key_encryption_key (
             ctx,
             TEST_BSON ("{'provider': 'azure', 'keyVaultEndpoint': "
                        "'example.vault.azure.net', 'keyName': 'test'}"));
          ASSERT_OK (mongocrypt_ctx_datakey_init (ctx), ctx);
-      } else if (0 == strcmp ("azure_oauth_decrypt", ctx_type)) {
+      } else if (0 == strcmp ("azure_oauth_decrypt", ctx_type)
+                 || 0 == strcmp ("azure_decrypt", ctx_type)) {
          bin = _mongocrypt_tester_encrypted_doc (tester);
          tester->paths.key_file = "./test/data/key-document-azure.json";
          ASSERT_OK (mongocrypt_ctx_decrypt_init (ctx, bin), ctx);
-      } else if (0 == strcmp ("gcp_oauth_datakey", ctx_type)) {
+      } else if (0 == strcmp ("gcp_oauth_datakey", ctx_type) ||
+                 0 == strcmp ("gcp_datakey", ctx_type)) {
          mongocrypt_ctx_setopt_key_encryption_key (
             ctx,
             TEST_BSON ("{'provider': 'gcp', 'projectId': 'test', 'location': "
                        "'global', 'keyRing': 'test', 'keyName': 'test'}"));
          ASSERT_OK (mongocrypt_ctx_datakey_init (ctx), ctx);
-      } else if (0 == strcmp ("gcp_oauth_decrypt", ctx_type)) {
+      } else if (0 == strcmp ("gcp_oauth_decrypt", ctx_type) ||
+                 0 == strcmp ("gcp_decrypt", ctx_type)) {
          bin = _mongocrypt_tester_encrypted_doc (tester);
          tester->paths.key_file = "./test/data/key-document-gcp.json";
          ASSERT_OK (mongocrypt_ctx_decrypt_init (ctx, bin), ctx);
@@ -96,6 +116,20 @@ _test_one_kms_response (_mongocrypt_tester_t *tester, bson_t *test)
       BSON_ASSERT (kms_ctx);
       BSON_ASSERT (mongocrypt_kms_ctx_bytes_needed (kms_ctx) > 0);
 
+      if (0 == strcmp ("gcp_datakey", ctx_type) ||
+          0 == strcmp ("gcp_decrypt", ctx_type) ||
+          0 == strcmp ("azure_datakey", ctx_type) ||
+          0 == strcmp ("azure_decrypt", ctx_type)) {
+         /* The targeted request is after the oauth request.
+          * Satisfy the oauth request with a valid response */
+         _satisfy_oauth_request (kms_ctx);
+         ASSERT_OK (mongocrypt_ctx_kms_done (ctx), ctx);
+         ASSERT_STATE_EQUAL (mongocrypt_ctx_state (ctx), MONGOCRYPT_CTX_NEED_KMS);
+         kms_ctx = mongocrypt_ctx_next_kms_ctx (ctx);
+         BSON_ASSERT (kms_ctx);
+         BSON_ASSERT (mongocrypt_kms_ctx_bytes_needed (kms_ctx) > 0);
+      }
+
       /* Feed until failure or completion. */
       BSON_ASSERT (bson_iter_init_find (&iter, test, "http_reply"));
       BSON_ASSERT (bson_iter_recurse (&iter, &iter));
@@ -121,7 +155,18 @@ _test_one_kms_response (_mongocrypt_tester_t *tester, bson_t *test)
 
    failed:
       BSON_ASSERT (bson_iter_init_find (&iter, test, "expect"));
-      expect = bson_iter_utf8 (&iter, NULL);
+      if (BSON_ITER_HOLDS_ARRAY (&iter)) {
+         // Concatenate array into one string.
+         bson_string_t *builder = bson_string_new (NULL);
+         bson_iter_recurse (&iter, &iter);
+         while (bson_iter_next (&iter)) {
+            ASSERT (BSON_ITER_HOLDS_UTF8 (&iter));
+            bson_string_append (builder, bson_iter_utf8 (&iter, NULL));
+         }
+         expect = bson_string_free (builder, false /* free segment */);
+      } else {
+         expect = bson_strdup (bson_iter_utf8 (&iter, NULL));
+      }
 
       if (0 == strcmp ("ok", expect)) {
          ASSERT_OR_PRINT (ok, status);
@@ -129,6 +174,7 @@ _test_one_kms_response (_mongocrypt_tester_t *tester, bson_t *test)
          ASSERT_STATUS_CONTAINS (status, expect);
       }
 
+      bson_free (expect);
       mongocrypt_binary_destroy (bin);
       mongocrypt_status_destroy (status);
       mongocrypt_ctx_destroy (ctx);
