@@ -551,9 +551,9 @@ _mongocrypt_tester_mongocrypt (tester_mongocrypt_flags flags)
    ASSERT_OK (mongocrypt_init (crypt), crypt);
    if (flags & TESTER_MONGOCRYPT_WITH_CRYPT_SHARED_LIB) {
       if (mongocrypt_crypt_shared_lib_version (crypt) == 0) {
-         BSON_ASSERT (
-            !"tester mongocrypt requested WITH_CRYPT_SHARED_LIB, but no "
-             "crypt_shared library was loaded by mongocrypt_init");
+         BSON_ASSERT (false &&
+                      "tester mongocrypt requested WITH_CRYPT_SHARED_LIB, but "
+                      "no crypt_shared library was loaded by mongocrypt_init");
       }
    }
    return crypt;
@@ -799,7 +799,12 @@ _test_setopt_invalid_kms_providers (_mongocrypt_tester_t *tester)
 
 typedef struct {
    char *value;
+   /* errmsg is the expected error message from mongocrypt_setopt_kms_providers
+    */
    char *errmsg;
+   /* errmsg_init is the expected error message from mongocrypt_init */
+   char *errmsg_init;
+   bool use_need_kms_credentials_state;
 } setopt_kms_providers_testcase_t;
 
 #define EXAMPLE_LOCAL_MATERIAL                                                 \
@@ -860,22 +865,42 @@ _test_setopt_kms_providers (_mongocrypt_tester_t *tester)
       {"{'kmip': {'endpoint': 'localhost' }}", NULL},
       {"{'kmip': {'endpoint': '127.0.0.1:5696', 'extra': 'invalid' }}",
        "Unexpected field: 'extra'"},
-      /* Empty documents are OK for on-demand KMS credentials */
-      {"{'aws': {}}", NULL},
-      {"{'azure': {}}", NULL},
-      {"{'local': {}}", NULL},
-      {"{'gcp': {}}", NULL},
-      {"{'kmip': {}}", NULL}};
+      /* Empty documents are OK if on-demand KMS credentials are opted-in with
+       * a call to mongocrypt_setopt_use_need_kms_credentials_state. */
+      {"{'aws': {}}", NULL, NULL, true},
+      {"{'azure': {}}", NULL, NULL, true},
+      {"{'local': {}}", NULL, NULL, true},
+      {"{'gcp': {}}", NULL, NULL, true},
+      {"{'kmip': {}}", NULL, NULL, true},
+      /* Empty documents are not OK if on-demand KMS credentials are not
+         opted-in. */
+      {"{'aws': {}}", NULL, "on-demand credentials not enabled", false},
+      {"{'azure': {}}", NULL, "on-demand credentials not enabled", false},
+      {"{'local': {}}", NULL, "on-demand credentials not enabled", false},
+      {"{'gcp': {}}", NULL, "on-demand credentials not enabled", false},
+      {"{'kmip': {}}", NULL, "on-demand credentials not enabled", false},
+      {"{'gcp': {'accessToken': 'foobar', 'email': 'foo@bar.com' }}",
+       "Unexpected field: 'email'"},
+      {.value = "{ 'azure': { 'accessToken': 'secret' } }"},
+   };
 
    for (i = 0; i < sizeof (tests) / sizeof (tests[0]); i++) {
       mongocrypt_t *crypt;
 
       test = tests + i;
       crypt = mongocrypt_new ();
+      if (test->use_need_kms_credentials_state) {
+         mongocrypt_setopt_use_need_kms_credentials_state (crypt);
+      }
       if (!test->errmsg) {
          ASSERT_OK (
             mongocrypt_setopt_kms_providers (crypt, TEST_BSON (test->value)),
             crypt);
+         if (!test->errmsg_init) {
+            ASSERT_OK (mongocrypt_init (crypt), crypt);
+         } else {
+            ASSERT_FAILS (mongocrypt_init (crypt), crypt, test->errmsg_init);
+         }
       } else {
          ASSERT_FAILS (
             mongocrypt_setopt_kms_providers (crypt, TEST_BSON (test->value)),
@@ -947,6 +972,13 @@ main (int argc, char **argv)
    _mongocrypt_tester_install_compact (&tester);
    _mongocrypt_tester_install_fle2_payload_uev (&tester);
    _mongocrypt_tester_install_fle2_payload_iup (&tester);
+   _mongocrypt_tester_install_range_encoding (&tester);
+   _mongocrypt_tester_install_range_edge_generation (&tester);
+   _mongocrypt_tester_install_range_mincover (&tester);
+   _mongocrypt_tester_install_mc_RangeOpts (&tester);
+   _mongocrypt_tester_install_mc_FLE2RangeFindDriverSpec (&tester);
+   _mongocrypt_tester_install_gcp_auth (&tester);
+   _mongocrypt_tester_install_mc_reader (&tester);
 
 #ifdef MONGOCRYPT_ENABLE_CRYPTO_COMMON_CRYPTO
    char osversion[32];
