@@ -228,6 +228,28 @@ _mongocrypt_tester_install_fle2_payload_uev (_mongocrypt_tester_t *tester);
 void
 _mongocrypt_tester_install_fle2_payload_iup (_mongocrypt_tester_t *tester);
 
+void
+_mongocrypt_tester_install_gcp_auth (_mongocrypt_tester_t *tester);
+
+void
+_mongocrypt_tester_install_range_encoding (_mongocrypt_tester_t *tester);
+
+void
+_mongocrypt_tester_install_range_edge_generation (_mongocrypt_tester_t *tester);
+
+void
+_mongocrypt_tester_install_range_mincover (_mongocrypt_tester_t *tester);
+
+void
+_mongocrypt_tester_install_mc_RangeOpts (_mongocrypt_tester_t *tester);
+
+void
+_mongocrypt_tester_install_mc_FLE2RangeFindDriverSpec (
+   _mongocrypt_tester_t *tester);
+
+void
+_mongocrypt_tester_install_mc_reader (_mongocrypt_tester_t *tester);
+
 /* Conveniences for getting test data. */
 
 /* Get a temporary bson_t from a JSON string. Do not free it. */
